#include "stdafx.h"
#include "CameraManager.h"

CameraManager *CameraManager::instance = NULL;

CameraManager *CameraManager::GetInstance()
{
	if (instance == NULL)
	{
		instance = new CameraManager();
	}
	return instance;
}

CameraManager::CameraManager()
{
}

CameraManager::~CameraManager()
{
}

/* Init the TUCAM API */
TUCAMRET CameraManager::InitApi()
{
	/* Get the current directory */
	m_itApi.uiCamCount = 0;
	m_itApi.pstrConfigPath = "./";

	TUCAM_Api_Init(&m_itApi);

	printf("Connect %d camera\r\n", m_itApi.uiCamCount);

	if (0 == m_itApi.uiCamCount)
	{
		return TUCAMRET_NO_CAMERA;
	}

	return TUCAMRET_SUCCESS;
}

/* UnInit the TUCAM API */
TUCAMRET CameraManager::UnInitApi()
{
	return TUCAM_Api_Uninit();
}

/* Open the camera by index number */
TUCAMRET CameraManager::OpenCamera(UINT uiIdx)
{
	if (uiIdx >= m_itApi.uiCamCount)
	{
		return TUCAMRET_OUT_OF_RANGE;
	}

	m_opCam.uiIdxOpen = uiIdx;

	return TUCAM_Dev_Open(&m_opCam);
}

/* Close the current camera */
TUCAMRET CameraManager::CloseCamera()
{
	if (NULL != m_opCam.hIdxTUCam)
	{
		TUCAM_Dev_Close(m_opCam.hIdxTUCam);
	}

	printf("Close the camera success\r\n");

	return TUCAMRET_SUCCESS;
}

char *CameraManager::GetCameraName()
{
	TUCAM_VALUE_INFO m_viCam; // Value info object
	m_viCam.nID = (int)TUIDI_CAMERA_MODEL;
	if (TUCAMRET_SUCCESS == TUCAM_Dev_GetInfo(m_opCam.hIdxTUCam, &m_viCam))
	{
		return m_viCam.pText;
	}
	return "";
}

int CameraManager::GetVID()
{
	TUCAM_VALUE_INFO m_viCam; // Value info object
	m_viCam.nID = (int)TUIDI_VENDOR;
	if (TUCAMRET_SUCCESS == TUCAM_Dev_GetInfo(m_opCam.hIdxTUCam, &m_viCam))
	{
		return m_viCam.nValue;
	}
	return 0;
}

int CameraManager::GetPID()
{
	TUCAM_VALUE_INFO m_viCam; // Value info object
	m_viCam.nID = (int)TUIDI_PRODUCT;
	if (TUCAMRET_SUCCESS == TUCAM_Dev_GetInfo(m_opCam.hIdxTUCam, &m_viCam))
	{
		return m_viCam.nValue;
	}
	return 0;
}

int CameraManager::GetCameraChannels()
{
	TUCAM_VALUE_INFO m_viCam; // Value info object
	m_viCam.nID = (int)TUIDI_CAMERA_CHANNELS;
	if (TUCAMRET_SUCCESS == TUCAM_Dev_GetInfo(m_opCam.hIdxTUCam, &m_viCam))
	{
		return m_viCam.nValue;
	}
	return 0;
}

char *CameraManager::GetUSBType()
{
	TUCAM_VALUE_INFO m_viCam; // Value info object
	m_viCam.nID = (int)TUIDI_BUS;
	if (TUCAMRET_SUCCESS == TUCAM_Dev_GetInfo(m_opCam.hIdxTUCam, &m_viCam))
	{
		if (0x200 == m_viCam.nValue || 0x210 == m_viCam.nValue)
		{
			return "2.0";
		}
		else
		{
			return "3.0";
		}
	}
	return "";
}

char *CameraManager::GetFirmwareVersion()
{
	TUCAM_VALUE_INFO m_viCam; // Value info object
	m_viCam.nID = (int)TUIDI_VERSION_FRMW;
	if (TUCAMRET_SUCCESS == TUCAM_Dev_GetInfo(m_opCam.hIdxTUCam, &m_viCam))
	{
		if (m_viCam.nValue != 0)
		{
			char *text = new char[50];
			sprintf_s(text, 50, "%x", m_viCam.nValue);
			return text;
		}
		else
		{
			return m_viCam.pText;
		}
	}
	return 0;
}

char *CameraManager::GetAPIVersion()
{
	TUCAM_VALUE_INFO m_viCam; // Value info object
	m_viCam.nID = (int)TUIDI_VERSION_API;
	if (TUCAMRET_SUCCESS == TUCAM_Dev_GetInfo(m_opCam.hIdxTUCam, &m_viCam))
	{
		return m_viCam.pText;
	}
	return "";
}