// C++04_prop_list.cpp : ̨Ӧóڵ㡣
//

#include "stdafx.h"
#include <conio.h> 
#include "../sdk/inc/TUCamApi.h"                         // SDKͷļ

#ifdef _WIN64
#pragma comment(lib, "../sdk/lib/x64/TUCam.lib")
#else
#pragma comment(lib, "../sdk/lib/x86/TUCam.lib")
#endif

static TUCAM_INIT m_itApi;       // SDK API initialized object
static TUCAM_OPEN m_opCam;       // Open camera object

static char m_strArrProp[][128] = { "TUIDP_GLOBALGAIN      : [Global Gain]                ",
"TUIDP_EXPOSURETM      : [Exposure Time]              ",
"TUIDP_BRIGHTNESS      : [Brightness(Auto Exposure)]  ",
"TUIDP_BLACKLEVEL      : [Black Level]                ",
"TUIDP_TEMPERATURE     : [Temperature]                ",
"TUIDP_SHARPNESS       : [Sharpness]                  ",
"TUIDP_NOISELEVEL      : [Noise Level]                ",
"TUIDP_HDR_KVALUE      : [HDR K Value]                ",
"TUIDP_GAMMA           : [Gamma]                      ",
"TUIDP_CONTRAST        : [Contrast]                   ",
"TUIDP_LFTLEVELS       : [Left Levels]                ",
"TUIDP_RGTLEVELS       : [Right Levels]               ",
"TUIDP_CHNLGAIN        : [Channel Gain]               ",
"TUIDP_SATURATION      : [Saturation]                 ",
"TUIDP_CLRTEMPERATURE  : [Color Temperature]          ",
"TUIDP_CLRMATRIX       : [Color Matrix]               ",
"TUIDP_DPCLEVEL        : [DPC Level]                  ",
"TUIDP_BLACKLEVELHG    : [Black Level High Gain]      ",
"TUIDP_BLACKLEVELLG    : [Black Level Low Gain]       ",
"TUIDP_POWEEFREQUENCY  : [Power frequency]            ",
"TUIDP_HUE             : [Hue]                        ",
"TUIDP_LIGHT           : [Light]                      ",
"TUIDP_ENHANCE_STRENGTH: [Enhance strength]           ",
"TUIDP_NOISELEVEL_3D   : [Noise Level 3D]             ",
"TUIDP_FOCUS_POSITION  : [Position]                   ",
"TUIDP_FRAME_RATE      : [Frame Rate]                 ",
"TUIDP_START_TIME      : [Start Time]                 ",
"TUIDP_FRAME_NUMBER    : [Frame Number]               ",
"TUIDP_INTERVAL_TIME   : [Interval Time]              ",
"TUIDP_GPS_APPLY       : [GPS Apply]                  ",
"TUIDP_AMB_TEMPERATURE : [Amb Temperature]            ",
"TUIDP_AMB_HUMIDITY    : [Amb Humidity]               ",
"TUIDP_AUTO_CTRLTEMP   : [Auto Control Temperature]   ",
"TUIDP_AVERAGEGRAY     : [Average]                    ",
"TUIDP_AVERAGEGRAYTHD  : [Average Gray Thd]           ",
"TUIDP_ENHANCETHD      : [Enhance Threshold ]         ",
"TUIDP_ENHANCEPARA     : [Enhance Parameter]          ",
"TUIDP_EXPOSUREMAX     : [Exposure Max]               ",
"TUIDP_EXPOSUREMIN     : [Exposure Min]               ",
"TUIDP_GAINMAX         : [Gain Max]                   ",
"TUIDP_GAINMIN         : [Gain Min]                   ",
"TUIDP_THROUGHFOGPARA  : [Through Fog]                ",
"TUIDP_ATLEVEL_PERCENTAGE   : [Auto Level Ignore]     ",
"TUIDP_TEMPERATURE_TARGET   : [Temperature Target]    ",
"TUIDP_PIXELRATIO      : [pixel ratio]                ",
};

/* Init the TUCAM API */
TUCAMRET InitApi()
{
	/* Get the current directory */
	m_itApi.uiCamCount = 0;
	m_itApi.pstrConfigPath = "./";

	TUCAM_Api_Init(&m_itApi);

	printf("Connect %d camera\r\n", m_itApi.uiCamCount);

	if (0 == m_itApi.uiCamCount)
	{
		return TUCAMRET_NO_CAMERA;
	}

	return TUCAMRET_SUCCESS;
}

/* UnInit the TUCAM API */
TUCAMRET UnInitApi()
{
	return TUCAM_Api_Uninit();
}

/* Open the camera by index number */
TUCAMRET OpenCamera(UINT uiIdx)
{
	if (uiIdx >= m_itApi.uiCamCount)
	{
		return TUCAMRET_OUT_OF_RANGE;
	}

	m_opCam.uiIdxOpen = uiIdx;

	return TUCAM_Dev_Open(&m_opCam);
}

/* Close the current camera */
TUCAMRET CloseCamera()
{
	if (NULL != m_opCam.hIdxTUCam)
	{
		TUCAM_Dev_Close(m_opCam.hIdxTUCam);
	}

	printf("Close the camera success\r\n");

	return TUCAMRET_SUCCESS;
}

/* Print the camera property list */
void PrintCameraPropertyList()
{
	double dbVal = 0;

	TUCAM_PROP_ATTR prop;
	prop.idProp = 0;
	prop.nIdxChn = 0;
	prop.dbValDft = 0;
	prop.dbValMin = 0;
	prop.dbValMax = 0;
	prop.dbValStep = 0;

	/* Get property list information */
	printf("Get property information list\r\n");

	for (int i = (int)TUIDP_GLOBALGAIN; i < (int)TUIDP_ENDPROPERTY; ++i)
	{
		prop.idProp = i;

		if (TUCAMRET_SUCCESS == TUCAM_Prop_GetAttr(m_opCam.hIdxTUCam, &prop))
		{
			printf("0x%02X.%s Range[%.2f, %.2f] Default:%.2f Step:%.2f\r\n", i, m_strArrProp[i], prop.dbValMin, prop.dbValMax, prop.dbValDft, prop.dbValStep);
		}
		else
		{
			printf("0x%02X.%s Not support\r\n", i, m_strArrProp[i]);
		}
	}

	printf("\r\n");

	/* Set property default value */
	printf("Set property default value\r\n");

	for (int i = (int)TUIDP_GLOBALGAIN; i < (int)TUIDP_ENDPROPERTY; ++i)
	{
		prop.idProp = i;
		prop.nIdxChn = 0;

		if (TUCAMRET_SUCCESS == TUCAM_Prop_GetAttr(m_opCam.hIdxTUCam, &prop))
		{

			if ((int)TUIDP_CLRTEMPERATURE == i
				|| (int)TUIDP_HDR_KVALUE == i
				|| (int)TUIDP_BLACKLEVELHG == i
				|| (int)TUIDP_BLACKLEVELLG == i)    /* Color Temperature*/
			{
				// Color temperature change the RGB value
				continue;
			}
			else if ((int)TUIDP_CHNLGAIN == i)     /* Channels Gain */
			{
				// Close the auto white balance state
				TUCAM_Capa_SetValue(m_opCam.hIdxTUCam, (int)TUIDC_ATWBALANCE, 0);

				// Red
				TUCAM_Prop_SetValue(m_opCam.hIdxTUCam, i, prop.dbValDft, 1);
				printf("0x%02X.%s Set default value %.2f success (channel red)\r\n", i, m_strArrProp[i], prop.dbValDft);

				// Green
				TUCAM_Prop_SetValue(m_opCam.hIdxTUCam, i, prop.dbValDft, 2);
				printf("0x%02X.%s Set default value %.2f success (channel green)\r\n", i, m_strArrProp[i], prop.dbValDft);

				// Blue
				TUCAM_Prop_SetValue(m_opCam.hIdxTUCam, i, prop.dbValDft, 3);
				printf("0x%02X.%s Set default value %.2f success (channel blue)\r\n", i, m_strArrProp[i], prop.dbValDft);
			}
			else
			{
				TUCAM_Prop_SetValue(m_opCam.hIdxTUCam, i, prop.dbValDft, 0);
				printf("0x%02X.%s Set default value %.2f success\r\n", i, m_strArrProp[i], prop.dbValDft);
			}
		}
	}

	printf("\r\n");

	/* Get property current value */
	printf("Get property current value\r\n");

	for (int i = (int)TUIDP_GLOBALGAIN; i < (int)TUIDP_ENDPROPERTY; ++i)
	{
		// Channels Gain
		if ((int)TUIDP_CHNLGAIN == i)
		{
			// Red
			if (TUCAMRET_SUCCESS == TUCAM_Prop_GetValue(m_opCam.hIdxTUCam, i, &dbVal, 1))
			{
				printf("0x%02X.%s The current value is %.2f success (channel red)\r\n", i, m_strArrProp[i], dbVal);
			}

			// Green
			if (TUCAMRET_SUCCESS == TUCAM_Prop_GetValue(m_opCam.hIdxTUCam, i, &dbVal, 2))
			{
				printf("0x%02X.%s The current value is %.2f success (channel green)\r\n", i, m_strArrProp[i], dbVal);
			}

			// Blue
			if (TUCAMRET_SUCCESS == TUCAM_Prop_GetValue(m_opCam.hIdxTUCam, i, &dbVal, 3))
			{
				printf("0x%02X.%s The current value is %.2f success (channel blue)\r\n", i, m_strArrProp[i], dbVal);
			}
		}
		else
		{
			if (TUCAMRET_SUCCESS == TUCAM_Prop_GetValue(m_opCam.hIdxTUCam, i, &dbVal, 0))
			{
				printf("0x%02X.%s The current value is %.2f\r\n", i, m_strArrProp[i], dbVal);
			}
		}
	}

	printf("\r\n");
}

int _tmain(int argc, _TCHAR* argv[])
{
	if (TUCAMRET_SUCCESS == InitApi())
	{
		if (TUCAMRET_SUCCESS == OpenCamera(0))
		{
			printf("Open the camera success\r\n");
			printf("\r\n");
			PrintCameraPropertyList();
			printf("\r\n");

			CloseCamera();
		}
		else
		{
			printf("Open the camera failure\r\n");
		}

		UnInitApi();
	}

	printf("Press any key to exit...\r\n");
	_getch();

	return 0;
}
