// C++10_trigger_software.cpp : ̨Ӧóڵ㡣
//

#include "stdafx.h"

#include <conio.h> 
#include "../sdk/inc/TUCamApi.h"                         // SDKͷļ

#ifdef _WIN64
#pragma comment(lib, "../sdk/lib/x64/TUCam.lib")
#else
#pragma comment(lib, "../sdk/lib/x86/TUCam.lib")
#endif

static TUCAM_INIT m_itApi;       // SDK API initialized object
static TUCAM_OPEN m_opCam;       // Open camera object
static TUCAM_FRAME m_frame;      // The frame object
static TUCAM_TRIGGER_ATTR m_tgr; // The trigger object

/* Init the TUCAM API */
TUCAMRET InitApi()
{
	/* Get the current directory */
	m_itApi.uiCamCount = 0;
	m_itApi.pstrConfigPath = "./";

	TUCAM_Api_Init(&m_itApi);

	printf("Connect %d camera\r\n", m_itApi.uiCamCount);

	if (0 == m_itApi.uiCamCount)
	{
		return TUCAMRET_NO_CAMERA;
	}

	return TUCAMRET_SUCCESS;
}

/* UnInit the TUCAM API */
TUCAMRET UnInitApi()
{
	return TUCAM_Api_Uninit();
}

/* Open the camera by index number */
TUCAMRET OpenCamera(UINT uiIdx)
{
	if (uiIdx >= m_itApi.uiCamCount)
	{
		return TUCAMRET_OUT_OF_RANGE;
	}

	m_opCam.uiIdxOpen = uiIdx;

	return TUCAM_Dev_Open(&m_opCam);
}

/* Close the current camera */
TUCAMRET CloseCamera()
{
	if (NULL != m_opCam.hIdxTUCam)
	{
		TUCAM_Dev_Close(m_opCam.hIdxTUCam);
	}

	printf("Close the camera success\r\n");

	return TUCAMRET_SUCCESS;
}

/* Do software trigger */
void DoSoftwareTrigger()
{
	int nTimes = 10;

	m_frame.pBuffer = NULL;
	m_frame.ucFormatGet = TUFRM_FMT_USUAl;
	m_frame.uiRsdSize = 1;

	TUCAM_Cap_GetTrigger(m_opCam.hIdxTUCam, &m_tgr);
	m_tgr.nTgrMode = (INT32)TUCCM_TRIGGER_SOFTWARE;
	m_tgr.nFrames = 1;
	TUCAM_Cap_SetTrigger(m_opCam.hIdxTUCam, m_tgr);

	TUCAM_Buf_Alloc(m_opCam.hIdxTUCam, &m_frame);

	long lRet = (long)TUCAMRET_NOT_SUPPORT;

	if (TUCAMRET_SUCCESS == TUCAM_Cap_Start(m_opCam.hIdxTUCam, (UINT32)TUCCM_TRIGGER_SOFTWARE))
	{
		for (int i = 0; i < nTimes; ++i)
		{
			/* Send software trigger signal */
			lRet = (long)TUCAM_Cap_DoSoftwareTrigger(m_opCam.hIdxTUCam);

			if ((long)TUCAMRET_SUCCESS == lRet)
			{
				if (TUCAMRET_SUCCESS == TUCAM_Buf_WaitForFrame(m_opCam.hIdxTUCam, &m_frame))
				{
					printf("Grab the software trigger frame success, index number is %d, width:%d, height:%d, channel:%d, depth:%d, image size:%d\r\n", i, m_frame.usWidth, m_frame.usHeight, m_frame.ucChannels, (2 == m_frame.ucElemBytes) ? 16 : 8, m_frame.uiImgSize);
				}
				else
				{
					printf("Grab the software trigger frame failure, index number is %d\r\n", i);
				}
			}
			else if ((long)TUCAMRET_NOT_SUPPORT == lRet)
			{
				printf("This camera cannot support software trigger\r\n");
				break;
			}
			else
			{
				printf("Send the software trigger signal failure\r\n");
			}
		}

		TUCAM_Buf_AbortWait(m_opCam.hIdxTUCam);
		TUCAM_Cap_Stop(m_opCam.hIdxTUCam);
	}

	TUCAM_Buf_Release(m_opCam.hIdxTUCam);
}

int _tmain(int argc, _TCHAR* argv[])
{
	if (TUCAMRET_SUCCESS == InitApi())
	{
		if (TUCAMRET_SUCCESS == OpenCamera(0))
		{
			printf("Open the camera success\r\n");
			printf("\r\n");
			DoSoftwareTrigger();
			printf("\r\n");
			CloseCamera();
		}
		else
		{
			printf("Open the camera failure\r\n");
		}

		UnInitApi();
	}

	printf("Press any key to exit...\r\n");
	_getch();

	return 0;
}

