#include "stdafx.h"
#include "CameraManager.h"
#include <stdio.h>

CameraManager *CameraManager::instance = NULL;

CameraManager *CameraManager::GetInstance()
{
	if (instance == NULL)
	{
		instance = new CameraManager();
	}
	return instance;
}

CameraManager::CameraManager()
{
}

CameraManager::~CameraManager()
{
}

/* Init the TUCAM API */
TUCAMRET CameraManager::InitApi()
{
	/* Get the current directory */
	m_itApi.uiCamCount = 0;
	m_itApi.pstrConfigPath = "./";

	TUCAM_Api_Init(&m_itApi);

	printf("Connect %d camera\r\n", m_itApi.uiCamCount);

	if (0 == m_itApi.uiCamCount)
	{
		return TUCAMRET_NO_CAMERA;
	}

	return TUCAMRET_SUCCESS;
}

/* UnInit the TUCAM API */
TUCAMRET CameraManager::UninitApi()
{
	return TUCAM_Api_Uninit();
}

/* Open the camera by index number */
TUCAMRET CameraManager::OpenCamera(UINT uiIdx)
{
	if (uiIdx >= m_itApi.uiCamCount)
	{
		return TUCAMRET_OUT_OF_RANGE;
	}

	m_opCam.uiIdxOpen = uiIdx;

	return TUCAM_Dev_Open(&m_opCam);
}

/* Close the current camera */
TUCAMRET CameraManager::CloseCamera()
{
	if (NULL != m_opCam.hIdxTUCam)
	{
		TUCAM_Dev_Close(m_opCam.hIdxTUCam);
	}

	printf("Close the camera success\r\n");

	return TUCAMRET_SUCCESS;
}

void CameraManager::SetAutoExposure()
{
	TUCAM_Capa_SetValue(m_opCam.hIdxTUCam, TUIDC_ATEXPOSURE, 1);
}

void CameraManager::SetExposure(double exposure)
{
	//رԶع
	int nVal;
	if (TUCAMRET_SUCCESS == TUCAM_Capa_GetValue(m_opCam.hIdxTUCam, TUIDC_ATEXPOSURE, &nVal))
	{
		if (nVal != 0)
		{
			printf("Close auto exposure.\n");
			TUCAM_Capa_SetValue(m_opCam.hIdxTUCam, TUIDC_ATEXPOSURE, 0);
		}
	}
	TUCAM_Prop_SetValue(m_opCam.hIdxTUCam, TUIDP_EXPOSURETM, exposure);
	printf("Set exposure %.2f\n", exposure);
}

/* Wait for image data */
void CameraManager::ShowAverageGray()
{
	int frameNumber = 1;

	m_frame.pBuffer = NULL;
	m_frame.ucFormatGet = TUFRM_FMT_USUAl;
	m_frame.uiRsdSize = 1;

	//TUCAM_Capa_SetValue(m_opCam.hIdxTUCam, TUIDC_ATEXPOSURE, 1);
	//Sleep(3000);
	TUCAM_Buf_Alloc(m_opCam.hIdxTUCam, &m_frame);

	if (TUCAMRET_SUCCESS == TUCAM_Cap_Start(m_opCam.hIdxTUCam, (UINT32)TUCCM_SEQUENCE))
	{
		for (int i = 0; i < frameNumber; ++i)
		{
			if (TUCAMRET_SUCCESS == TUCAM_Buf_WaitForFrame(m_opCam.hIdxTUCam, &m_frame))
			{
				printf("Grab the frame success, index number is %d, width:%d, height:%d, channel:%d, depth:%d, image size:%d\r\n", i, m_frame.usWidth, m_frame.usHeight, m_frame.ucChannels, (2 == m_frame.ucElemBytes) ? 16 : 8, m_frame.uiImgSize);
				printf("Average Gray:%d\r\n", CalculateAverageGray(m_frame));		
			}
			else
			{
				printf("Grab the frame failure, index number is %d\r\n", i);
			}
		}

		TUCAM_Buf_AbortWait(m_opCam.hIdxTUCam);
		TUCAM_Cap_Stop(m_opCam.hIdxTUCam);
	}

	TUCAM_Buf_Release(m_opCam.hIdxTUCam);
}

int CameraManager::CalculateAverageGray(const TUCAM_FRAME &frame)
{
	if (frame.pBuffer == NULL)
	{
		return 0;
	}

	int size = frame.usWidth * frame.usHeight;
	ULONG64 sum = 0;
	UINT averageGray = 0;

	if (1 == frame.ucChannels) // Gray Image
	{
		if (2 == frame.ucElemBytes) // 16bit
		{
			USHORT *imageData = (USHORT*)(frame.pBuffer + frame.usHeader);
			for (int i = 0; i < size; ++i)
			{
				sum += *imageData;
				imageData++;
			}
			averageGray = sum / size;
			averageGray = min(averageGray, 65535);
		}
		else // 8bit
		{
			UCHAR *imageData = frame.pBuffer + frame.usHeader;
			for (int i = 0; i < size; ++i)
			{
				sum += *imageData;
				imageData++;
			}
			averageGray = sum / size;
			averageGray = min(averageGray, 255);
		}
	}
	else if (3 == frame.ucChannels) // Color Image
	{
		int gray;
		if (2 == frame.ucElemBytes)
		{
			USHORT *imageData = (USHORT*)(frame.pBuffer + frame.usHeader);
			for (int i = 0; i < size; ++i)
			{
				gray = *imageData * 0.114 + *(imageData + 1) * 0.587 + *(imageData + 2) * 0.299; // Calculate gray value;
				sum += gray;
				imageData += 3;
			}
			averageGray = sum / size;
			averageGray = min(averageGray, 65535);
		}
		else
		{
			UCHAR *imageData = frame.pBuffer + frame.usHeader;
			for (int i = 0; i < size; ++i)
			{
				gray = *imageData * 0.114 + *(imageData + 1) * 0.587 + *(imageData + 2) * 0.299;
				sum += gray;
				imageData += 3;
			}
			averageGray = sum / size;
			averageGray = min(averageGray, 255);
		}
	}

	return averageGray;
}

void CameraManager::GetParameter()
{
	TUCAM_ELEMENT node;
	node.pName = (char *)"Root";

	char access[][3] = { "NI", "NA", "WO", "RO", "RW" };
	char elemType[][20] = {
		"Value",
		"Base",
		"Integer",
		"Boolean",
		"Command",
		"Float",
		"String",
		"Register",
		"Category",
		"Enumeration",
		"EnumEntry",
		"Port"
	};
	while (TUCAMRET_SUCCESS == TUCAM_GenICam_ElementAttrNext(m_opCam.hIdxTUCam, &node, node.pName, TU_CAMERA_XML))
	{
		if (node.pName == NULL)
		{
			continue;
		}
		switch (node.Type)
		{
		case TU_ElemCategory:
			printf("level: %d, name: %s\n", node.Level, node.pName);
			break;
		case TU_ElemBoolean:
		case TU_ElemCommand:
		case TU_ElemInteger:
			printf("level: %d, asscess: %s, type: %s, name: %s, value: %d\n", node.Level, access[node.Access], elemType[node.Type], node.pName, node.nVal);
			break;
		case TU_ElemFloat:
			printf("level: %d, asscess: %s, type: %s, name: %s, value: %f\n", node.Level, access[node.Access], elemType[node.Type], node.pName, node.dbVal);
			break;
		case TU_ElemString:
			node.pTransfer = new char[node.nMax];
			memset(node.pTransfer, 0, node.nMax);
			TUCAM_GenICam_GetElementValue(m_opCam.hIdxTUCam, &node, TU_CAMERA_XML);
			printf("level: %d, asscess: %s, type: %s, name: %s, value: %s\n", node.Level, access[node.Access], elemType[node.Type], node.pName, node.pTransfer);
			break;
		case TU_ElemEnumeration:
			printf("level: %d, asscess: %s, type: %s, name: %s, value: %s\n", node.Level, access[node.Access], elemType[node.Type], node.pName, node.pEntries[node.nVal]);
			break;
		case TU_ElemRegister:
			printf("level: %d, asscess: %s, type: %s, name: %s\n", node.Level, access[node.Access], elemType[node.Type], node.pName);
			break;
		}
	}
	printf("GetParameter finish.\n");
}

void CameraManager::SetParameter()
{
	GetParameter();
	printf("\n\nSetParameter finish.\n");
	SetParameter("DeviceUserID", TU_ElemString, 0, "Dhyana 2100 Pro");
	SetParameter("AnalogGain", TU_ElemEnumeration, 1, "");
	SetParameter("ExposureTime", TU_ElemFloat, 15.0, "");
	SetParameter("TriggerFrameNum", TU_ElemInteger, 10, "");
	SetParameter("BlackLevel", TU_ElemInteger, 10, "");
	SetParameter("ReverseX", TU_ElemBoolean, 0, "");
	GetParameter();
	printf("\n\nSetParameter finish.\n");
	SetParameter("DeviceUserID", TU_ElemString, 0, "Dhyana 2100");
	SetParameter("AnalogGain", TU_ElemEnumeration, 0, "");
	SetParameter("ExposureTime", TU_ElemFloat, 5.0, "");
	SetParameter("TriggerFrameNum", TU_ElemInteger, 1, "");
	SetParameter("BlackLevel", TU_ElemInteger, 100, "");
	SetParameter("ReverseX", TU_ElemBoolean, 1, "");
	GetParameter();
	printf("SetParameter finish.\n");
}

void CameraManager::SetParameter(const char *name, int type, double value, const char *stringValue)
{
	TUCAM_ELEMENT node;
	node.pName = new char[100];
	strcpy_s(node.pName, 100, name);
	
	TUCAM_GenICam_ElementAttr(m_opCam.hIdxTUCam, &node, node.pName, TU_CAMERA_XML);
	switch (node.Type)
	{
	case TU_ElemCategory:
		printf("level: %d, name: %s\n", node.Level, node.pName);
		break;
	case TU_ElemBoolean:
	case TU_ElemCommand:
	case TU_ElemInteger:
		if (node.Type != TU_ElemFloat)
		{
			node.nVal = value;
		}
	case TU_ElemFloat:
		if (node.Type == TU_ElemFloat)
		{
			node.dbVal = value;
		}
		TUCAM_GenICam_SetElementValue(m_opCam.hIdxTUCam, &node, TU_CAMERA_XML);
		break;
	case TU_ElemString:
	case TU_ElemRegister:
		node.pTransfer = new char[node.nMax];
		strcpy_s(node.pTransfer, node.nMax, stringValue);
		TUCAM_GenICam_SetElementValue(m_opCam.hIdxTUCam, &node, TU_CAMERA_XML);
		break;
	case TU_ElemEnumeration:
		node.nVal = min(node.nMax, max(0, value));
		TUCAM_GenICam_SetElementValue(m_opCam.hIdxTUCam, &node, TU_CAMERA_XML);
		break;
	}
	//delete node.pName;
}

void CameraManager::CallBack()
{
	TUCAM_Buf_DataCallBack(m_opCam.hIdxTUCam, OnCallbackBuffer, this);
	StartCapture();
	TUCAM_Buf_WaitForFrame(m_opCam.hIdxTUCam, &m_frame, 1000);
	Sleep(5000);
	StopCapture();
}

void CameraManager::OnCallbackBuffer(void *userContext)
{
	CameraManager *manager = (CameraManager *)userContext;
	manager->GetData();
}

void CameraManager::GetData()
{
	TUCAM_RAWIMG_HEADER frame;
	int result = TUCAM_Buf_GetData(m_opCam.hIdxTUCam, &frame);
	printf("TUCAM_Buf_GetData = %d \n", result);
	printf("index: %d, imageSize: %d, width: %d, height: %d\n", frame.uiIndex, frame.uiImgSize, frame.usWidth, frame.usHeight);

	printf("index: %d, time stamp: %f, time last: %f\n", frame.uiIndex, frame.dblTimeStamp, frame.dblTimeLast);
	unsigned char *imageData = new unsigned char[frame.uiImgSize];
	memcpy(imageData, frame.pImgData, frame.uiImgSize);
	SaveData("e://_image.raw", imageData, frame.uiImgSize);
	delete[] imageData;
}

void CameraManager::SaveData(const char *path, unsigned char *data, int size)
{
	FILE *file = fopen(path, "wb");
	if (file == NULL)
	{
		printf("SaveData failed.\n");
		return;
	}
	fwrite(data, sizeof(unsigned char), size, file);
	fclose(file);
}

void CameraManager::StartCapture()
{
	m_frame.pBuffer = 0;
	m_frame.ucFormatGet = TUFRM_FMT_RAW;
	m_frame.uiRsdSize = 1;

	TUCAM_Buf_Alloc(m_opCam.hIdxTUCam, &m_frame);
	TUCAM_Cap_Start(m_opCam.hIdxTUCam, TUCCM_SEQUENCE);
}

void CameraManager::StopCapture()
{
	TUCAM_Buf_AbortWait(m_opCam.hIdxTUCam);
	TUCAM_Cap_Stop(m_opCam.hIdxTUCam);
	TUCAM_Buf_Release(m_opCam.hIdxTUCam);
}

void CameraManager::Binning()
{
	TUCAM_CAPA_ATTR capaAttr;
	capaAttr.idCapa = TUIDC_BINNING_SUM;
	TUCAM_Capa_GetAttr(m_opCam.hIdxTUCam, &capaAttr);
	printf("CapaID = %d, min = %d, max = %d, default = %d\n", capaAttr.idCapa, capaAttr.nValMin, capaAttr.nValMax, capaAttr.nValDft);

	TUCAM_Capa_SetValue(m_opCam.hIdxTUCam, TUIDC_BINNING_SUM, capaAttr.nValMax);
	int value;
	TUCAM_Capa_GetValue(m_opCam.hIdxTUCam, TUIDC_BINNING_SUM, &value);
	printf("Set max bin = %d\n", value);	
	ShowAverageGray();

	TUCAM_Capa_SetValue(m_opCam.hIdxTUCam, TUIDC_BINNING_SUM, capaAttr.nValDft);
	TUCAM_Capa_GetValue(m_opCam.hIdxTUCam, TUIDC_BINNING_SUM, &value);
	printf("Set default bin = %d\n", value);
	ShowAverageGray();
}

void CameraManager::Histogram()
{
	int frameNumber = 1;

	m_frame.pBuffer = NULL;
	m_frame.ucFormatGet = TUFRM_FMT_USUAl;
	m_frame.uiRsdSize = 1;

	TUCAM_Buf_Alloc(m_opCam.hIdxTUCam, &m_frame);

	if (TUCAMRET_SUCCESS == TUCAM_Cap_Start(m_opCam.hIdxTUCam, (UINT32)TUCCM_SEQUENCE))
	{
		TUCAM_Capa_SetValue(m_opCam.hIdxTUCam, TUIDC_HISTC, 1);
		for (int i = 0; i < frameNumber; ++i)
		{
			if (TUCAMRET_SUCCESS == TUCAM_Buf_WaitForFrame(m_opCam.hIdxTUCam, &m_frame))
			{
				printf("Grab the frame success, index number is %d, width:%d, height:%d, channel:%d, depth:%d, image size:%d\r\n", i, m_frame.usWidth, m_frame.usHeight, m_frame.ucChannels, (2 == m_frame.ucElemBytes) ? 16 : 8, m_frame.uiImgSize);
				printf("Average Gray:%d\r\n", CalculateAverageGray(m_frame));

				int histogram[65536];
				int histogramSize = 65536;
				int *histogramSrc = (int *)(m_frame.pBuffer + m_frame.usHeader + m_frame.uiImgSize);
				if (m_frame.ucChannels == 1)
				{
					for (int i = 0; i < histogramSize; ++i)
					{
						histogram[i] = histogramSrc[i];
					}
				}
				else
				{
					int currentChannel = 3;					
					TUCAM_Capa_SetValue(m_opCam.hIdxTUCam, TUIDC_CHANNELS, currentChannel);
					histogramSrc = histogramSrc + currentChannel * histogramSize;
					for (int i = 0; i < histogramSize; ++i)
					{
						histogram[i] = histogramSrc[i];
					}
				}

				int currentChannel;
				TUCAM_Capa_GetValue(m_opCam.hIdxTUCam, TUIDC_CHANNELS, &currentChannel);
				freopen("histogram.txt", "w", stdout);
				printf("histogram of channel %d\n", currentChannel);
				for (int i = 0; i < histogramSize; ++i)
				{
					printf("gray = %d, count = %d\n", i, histogram[i]);
				}
				fclose(stdout);
				WinExec("notepad.exe histogram.txt", SW_SHOWNORMAL);
			}
			else
			{
				printf("Grab the frame failure, index number is %d\r\n", i);
			}
		}

		TUCAM_Buf_AbortWait(m_opCam.hIdxTUCam);
		TUCAM_Cap_Stop(m_opCam.hIdxTUCam);
	}

	TUCAM_Buf_Release(m_opCam.hIdxTUCam);
}

void CameraManager::AutoLevel()
{
	m_frame.pBuffer = NULL;
	m_frame.ucFormatGet = TUFRM_FMT_USUAl;
	m_frame.uiRsdSize = 1;

	TUCAM_Buf_Alloc(m_opCam.hIdxTUCam, &m_frame);

	if (TUCAMRET_SUCCESS == TUCAM_Cap_Start(m_opCam.hIdxTUCam, (UINT32)TUCCM_SEQUENCE))
	{
		printf("Set histogram enable\n");
		// 1 Set histogram enable
		TUCAM_Capa_SetValue(m_opCam.hIdxTUCam, TUIDC_HISTC, 1);

		// 2 Set auto level 1:auto left, level 2 : auto right, level 3 : auto level
		printf("Set auto level");
		TUCAM_Capa_SetValue(m_opCam.hIdxTUCam, TUIDC_ATLEVELS, 3);
		
		//3 Set auto level percentage only support mono camera 10->0.001
		printf("Set auto level percentage\n");
		TUCAM_Prop_SetValue(m_opCam.hIdxTUCam, TUIDP_ATLEVEL_PERCENTAGE, 10, 0);
		int frameNumber = 100;
		for (int i = 0; i < frameNumber; ++i)
		{
			if (TUCAMRET_SUCCESS == TUCAM_Buf_WaitForFrame(m_opCam.hIdxTUCam, &m_frame))
			{
				printf("Grab the frame success, index number is %d, width:%d, height:%d, channel:%d, depth:%d, image size:%d\r\n", i, m_frame.usWidth, m_frame.usHeight, m_frame.ucChannels, (2 == m_frame.ucElemBytes) ? 16 : 8, m_frame.uiImgSize);
				printf("Average Gray:%d\r\n", CalculateAverageGray(m_frame));

				double leftLevel, rightLevel;
				TUCAM_Capa_SetValue(m_opCam.hIdxTUCam, TUIDC_CHANNELS, 0);
				int currentChannel;
				TUCAM_Capa_GetValue(m_opCam.hIdxTUCam, TUIDC_CHANNELS, &currentChannel);
				printf("histogram of channel %d\n", currentChannel);
				TUCAM_Prop_GetValue(m_opCam.hIdxTUCam, TUIDP_LFTLEVELS, &leftLevel, currentChannel);
				TUCAM_Prop_GetValue(m_opCam.hIdxTUCam, TUIDP_RGTLEVELS, &rightLevel, currentChannel);

				printf("left level = %.0f, right level = %.0f\n", leftLevel, rightLevel);
			}
			else
			{
				printf("Grab the frame failure, index number is %d\r\n", i);
			}
		}

		TUCAM_Buf_AbortWait(m_opCam.hIdxTUCam);
		TUCAM_Cap_Stop(m_opCam.hIdxTUCam);
	}

	TUCAM_Buf_Release(m_opCam.hIdxTUCam);
}

void CameraManager::AreaWhiteBalance()
{
	ShowAverageGray();
	AreaWhiteBalance(1); // Enable area white balance

	Sleep(1500);
	AreaWhiteBalance(0); // Disable area white balance

	ShowAverageGray();
}

void CameraManager::AreaWhiteBalance(bool value)
{
	TUCAM_CALC_ROI_ATTR roi;
	roi.bEnable = value;
	roi.idCalc = TUIDCR_WBALANCE;
	roi.nHOffset = 0;
	roi.nVOffset = 0;
	roi.nWidth = 320;
	roi.nHeight = 240;
	TUCAM_Calc_SetROI(m_opCam.hIdxTUCam, roi);

	printf("Set = %d area white balance success, HOffset:%d, VOffset:%d, Width:%d, Height:%d\n", roi.bEnable, roi.nHOffset, roi.nVOffset, roi.nWidth, roi.nHeight);

}

void CameraManager::TimeStamp()
{
	CallBack();
	StartCapture();
	int frameNumber = 10;
	for (int i = 0; i < frameNumber; ++i)
	{
		if (TUCAMRET_SUCCESS == TUCAM_Buf_WaitForFrame(m_opCam.hIdxTUCam, &m_frame))
		{
			printf("Grab the frame success, index number is %d, width:%d, height:%d, channel:%d, depth:%d, image size:%d\r\n", i, m_frame.usWidth, m_frame.usHeight, m_frame.ucChannels, (2 == m_frame.ucElemBytes) ? 16 : 8, m_frame.uiImgSize);
			printf("Average Gray:%d\r\n", CalculateAverageGray(m_frame));

			TUCAM_IMG_HEADER header;
			memcpy(&header, m_frame.pBuffer, m_frame.usHeader);
			printf("index: %d, time stamp: %f, time last: %f\n", header.uiIndex, header.dblTimeStamp, header.dblTimeLast);
		}
		else
		{
			printf("Grab the frame failure, index number is %d\r\n", i);
		}
	}
	StopCapture();
}

void CameraManager::GpsTrigger()
{
	TUCAM_TRIGGER_ATTR trigger;
	TUCAM_Cap_GetTrigger(m_opCam.hIdxTUCam, &trigger);
	trigger.nTgrMode = TUCCM_TRIGGER_GPS;
	trigger.nBufFrames = 1;
	TUCAM_Cap_SetTrigger(m_opCam.hIdxTUCam, trigger);
	
	// Set GPS parameter
	// GPS Start Time dwTime = (m_nHour << 16) + (m_nMin << 8) + m_nSec;
	TUCAM_Prop_SetValue(m_opCam.hIdxTUCam, TUIDP_START_TIME, 0);
	// GPS Frame Number
	TUCAM_Prop_SetValue(m_opCam.hIdxTUCam, TUIDP_FRAME_NUMBER, 1);
	// GPS Interval Time
	TUCAM_Prop_SetValue(m_opCam.hIdxTUCam, TUIDP_INTERVAL_TIME, 1);

	// Get GPS parameter
	TUCAM_VALUE_INFO info;
	info.nID = TUIDI_UTCTIME;
	TUCAM_Dev_GetInfo(m_opCam.hIdxTUCam, &info);
	
	printf("UTC Reference Time : %s \n", info.pText);
	
	info.nID = TUIDI_UTCTIME;
	TUCAM_Dev_GetInfo(m_opCam.hIdxTUCam, &info);		
	printf("UTC pos : %s", info.pText);

	StartCapture();
	int frameNumber = 10;
	for (int i = 0; i < frameNumber; ++i)
	{
		if (TUCAMRET_SUCCESS == TUCAM_Buf_WaitForFrame(m_opCam.hIdxTUCam, &m_frame))
		{
			printf("Grab the frame success, index number is %d, width:%d, height:%d, channel:%d, depth:%d, image size:%d\r\n", i, m_frame.usWidth, m_frame.usHeight, m_frame.ucChannels, (2 == m_frame.ucElemBytes) ? 16 : 8, m_frame.uiImgSize);
			printf("Average Gray:%d\r\n", CalculateAverageGray(m_frame));

			TUCAM_IMG_HEADER header;
			memcpy(&header, m_frame.pBuffer, m_frame.usHeader);
			printf("Get the gps parameter success, index number is %d, Year:%d, Month:%d, Day:%d, Hour:%d, Min:%d, Sec:%d, Ns:%d\n", i, header.ucGPSTimeStampYear, header.ucGPSTimeStampMonth, header.ucGPSTimeStampDay, header.ucGPSTimeStampHour, header.ucGPSTimeStampMin, header.ucGPSTimeStampSec, header.nGPSTimeStampNs);
		}
		else
		{
			printf("Grab the frame failure, index number is %d\r\n", i);
		}
	}
	StopCapture();
}

int CameraManager::GetResolutionCount()
{
	TUCAM_CAPA_ATTR capaAttr;
	capaAttr.idCapa = TUIDC_RESOLUTION;

	if (TUCAMRET_SUCCESS == TUCAM_Capa_GetAttr(m_opCam.hIdxTUCam, &capaAttr))
	{
		return capaAttr.nValMax - capaAttr.nValMin + 1;
	}

	return 0;
}

void CameraManager::SetResolution(int index)
{
	TUCAM_Capa_SetValue(m_opCam.hIdxTUCam, TUIDC_RESOLUTION, index);
}

void CameraManager::OpenStream()
{
	m_frame.pBuffer = NULL;
	m_frame.ucFormatGet = TUFRM_FMT_USUAl;
	m_frame.uiRsdSize = 1;
	TUCAM_Buf_Alloc(m_opCam.hIdxTUCam, &m_frame);

	if (TUCAMRET_SUCCESS == TUCAM_Cap_Start(m_opCam.hIdxTUCam, (UINT32)TUCCM_SEQUENCE))
	{
		printf("TUCAM_Cap_Start succeeded.\r\n");
	}
	else
	{
		printf("TUCAM_Cap_Start failed.\r\n");
	}
}

void CameraManager::CloseStream()
{
	TUCAM_Buf_AbortWait(m_opCam.hIdxTUCam);
	TUCAM_Cap_Stop(m_opCam.hIdxTUCam);
	TUCAM_Buf_Release(m_opCam.hIdxTUCam);
}

void CameraManager::TestCalculateHistogramData()
{
	int nTimes = 10;

	m_frame.pBuffer = NULL;
	m_frame.ucFormatGet = TUFRM_FMT_USUAl;
	m_frame.uiRsdSize = 1;

	TUCAM_Capa_SetValue(m_opCam.hIdxTUCam, TUIDC_ATEXPOSURE, 1);
	Sleep(3000);
	TUCAM_Buf_Alloc(m_opCam.hIdxTUCam, &m_frame);

	if (TUCAMRET_SUCCESS == TUCAM_Cap_Start(m_opCam.hIdxTUCam, (UINT32)TUCCM_SEQUENCE))
	{
		for (int i = 0; i < nTimes; ++i)
		{
			if (TUCAMRET_SUCCESS == TUCAM_Buf_WaitForFrame(m_opCam.hIdxTUCam, &m_frame))
			{
				printf("Grab the frame success, index number is %d, width:%d, height:%d, channel:%d, depth:%d, image size:%d\r\n", i, m_frame.usWidth, m_frame.usHeight, m_frame.ucChannels, (2 == m_frame.ucElemBytes) ? 16 : 8, m_frame.uiImgSize);
				CalculateHistogramData(m_frame);
			}
			else
			{
				printf("Grab the frame failure, index number is %d\r\n", i);
			}
		}

		TUCAM_Buf_AbortWait(m_opCam.hIdxTUCam);
		TUCAM_Cap_Stop(m_opCam.hIdxTUCam);
	}

	TUCAM_Buf_Release(m_opCam.hIdxTUCam);
}

void CameraManager::CalculateHistogramData(const TUCAM_FRAME &frame)
{
	if (frame.pBuffer == NULL)
	{
		return;
	}

	int count = 256; // 8bit image
	if (2 == frame.ucElemBytes) // 16bit image
	{
		count = 65536;
	}
	int *data = new int[count]; // histogram data array
	for (int i = 0; i < count; ++i)
	{
		data[i] = 0;
	}

	int size = frame.usWidth * frame.usHeight;
	if (1 == frame.ucChannels) // Gray Image
	{
		if (2 == frame.ucElemBytes)
		{
			USHORT *imageData = (USHORT*)(frame.pBuffer + frame.usHeader); // image data point
			for (int i = 0; i < size; ++i)
			{
				++data[*imageData]; // calculate histogram data
				imageData++; // next pixel
			}
		}
		else
		{
			UCHAR *imageData = frame.pBuffer + frame.usHeader;
			for (int i = 0; i < size; ++i)
			{
				++data[*imageData];
				imageData++;
			}
		}
	}
	else if (3 == frame.ucChannels) // Color Image
	{
		int gray;
		if (2 == frame.ucElemBytes)
		{
			USHORT *imageData = (USHORT*)(frame.pBuffer + frame.usHeader);
			for (int i = 0; i < size; ++i)
			{
				gray = *imageData * 0.114 + *(imageData + 1) * 0.587 + *(imageData + 2) * 0.299; // Calculate gray value;
				++data[gray];
				imageData += 3;
			}
		}
		else
		{
			UCHAR *imageData = frame.pBuffer + frame.usHeader;
			for (int i = 0; i < size; ++i)
			{
				gray = *imageData * 0.114 + *(imageData + 1) * 0.587 + *(imageData + 2) * 0.299;
				++data[gray];
				imageData += 3;
			}
		}
	}
	for (int i = 0; i < count; ++i)
	{
		printf("gray : %8d , count : %12d\n", i, data[i]);
	}
}