// CameraControl.cpp : implementation file
//

#include "stdafx.h"
#include "SamplePro.h"
#include "CameraControl.h"
#include "Trigger.h"
#include "ROI.h"
//#include "afxdialogex.h"

#define IDT_GET_TEMPERATURE     0x64        
#define IDT_SPC_TEMPERATURE     1000	// Interval 1 Second
#define IDT_GET_FPS_RATE        0x68
#define IDT_SPC_FPS_RATE        1000	// Interval 1 Second
#define IDT_GET_ONCE_AE         0x69
#define IDT_SPC_ONCE_AE         500		// Interval 0.5 Second
#define IDT_GET_AE				0x70
#define IDT_SPC_AE				1000	// Interval 1 Second
// CameraControl dialog

IMPLEMENT_DYNAMIC(CameraControl, CDialog)

CameraControl::CameraControl(TUCAM_OPEN &opCam,CCameraControlMode* pParent)
	: CDialog(CameraControl::IDD, pParent)
	, m_nGain(0)
	, m_dbFrameRate(0)
{
	m_opCam                 = opCam;
	m_ccmode                = pParent;
	m_uiExpStep             = 1;
	m_bRatioRange           = FALSE;
	m_bTargetRange          = FALSE;
	m_uSec					= 0;
	m_uMs					= 0;
	m_uUs					= 0;
	m_bFirst                = TRUE;
	m_ucount                = 0;
	m_favg                  = 0.0;
	m_ffast                 = 0.0;
	m_fslow                 = 0.0;
	m_bGainRange            = TRUE;
    m_bSprtCMS              = FALSE;

	m_bAlarmState           = FALSE;
	m_bMessageBoxShow       = FALSE;

    m_bLedState             = FALSE;
	m_bPIState              = FALSE;
	m_bDSNUState            = FALSE;
	m_bPRNUState            = FALSE;
	m_bSupportWaterCool     = FALSE;
    m_nImgModeMin           = 0;
	m_nImgModeMax           = 0;
	m_bTEC                  = FALSE;
	m_nMidTTemp             = 0;
	m_bTempChange           = FALSE;
	m_nPID                  = m_ccmode->m_nPID;

	GlobalAttachObserver(this);
 }

CameraControl::~CameraControl()
{
//  SaveProfile();
	GlobalDetachObserver(this);
}

void CameraControl::UpdateObserver(int message, void *param)
{
	/*if (message == MESSAGE_UPDATE_FPS)
	{
		float fps = *((float *)param);
		PostMessage(TUMSG_FPS, NULL, (LPARAM)fps);
	}
	else*/ if (message == MESSAGE_UPDATE_EXPOSURE_TIME)
	{
		double expTime = *((double *)param);
		UpdateExposureTime(expTime);
	}
}
void CameraControl::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_CMB_RES, m_cmbRes);
	DDX_Control(pDX, IDC_CHK_BITOFDEPTH, m_chkBDep);
	DDX_Control(pDX, IDC_CHK_AE, m_chkAE);
	DDX_Control(pDX, IDC_SLD_TGR, m_sldTgr);
	DDX_Control(pDX, IDC_SLD_GAIN, m_sldGain);
	DDX_Text(pDX, IDC_EDT_SEC, m_uSec);
	DDX_Text(pDX, IDC_EDT_MS, m_uMs);
	DDX_Text(pDX, IDC_EDT_US, m_uUs);
	DDX_Control(pDX, IDC_CMB_BITOFDEPTH, m_cmbBitOfDepth);
	DDX_Control(pDX, IDC_CMB_TESTIMG, m_cmbTestImg);
	DDX_Text(pDX, IDC_EDT_RATE, m_dbFrameRate);
	DDX_Control(pDX, IDC_CMB_FAN, m_cmbFan);
	DDX_Control(pDX, IDC_SLD_COOLTARGET, m_sldCoolT);
	DDX_Control(pDX, IDC_CMB_GAIN, m_cmbGain);
	DDX_Control(pDX, IDC_CMB_SHUTTER, m_cmbShutter);
	DDX_Control(pDX, IDC_CHK_TIMESTAMP, m_chkTimestamp);
	DDX_Control(pDX, IDC_CMB_IMGSELECT, m_cmbImgSelect);
	DDX_Control(pDX, IDC_SLD_RATIO, m_sldRatio);
}


BEGIN_MESSAGE_MAP(CameraControl, CDialog)
	ON_BN_CLICKED(IDC_BTN_LIVE, &CameraControl::OnBnClickedBtnLive)
	ON_CBN_SELCHANGE(IDC_CMB_RES, &CameraControl::OnCbnSelchangeCmbRes)
	ON_BN_CLICKED(IDC_CHK_AE, &CameraControl::OnBnClickedChkAe)
	ON_NOTIFY(NM_CUSTOMDRAW, IDC_SLD_TGR, &CameraControl::OnNMCustomdrawSldTgr)
	ON_NOTIFY(UDN_DELTAPOS, IDC_SPN_SEC, &CameraControl::OnDeltaposSpnSec)
	ON_NOTIFY(UDN_DELTAPOS, IDC_SPN_MS, &CameraControl::OnDeltaposSpnMs)
	ON_NOTIFY(UDN_DELTAPOS, IDC_SPN_US, &CameraControl::OnDeltaposSpnUs)
	ON_NOTIFY(NM_CUSTOMDRAW, IDC_SLD_GAIN, &CameraControl::OnNMCustomdrawSldGain)
	ON_BN_CLICKED(IDC_BTN_EXPTM, &CameraControl::OnBnClickedBtnExptm)
	ON_BN_CLICKED(IDC_CHK_BITOFDEPTH, &CameraControl::OnBnClickedChkBitofdepth)
	ON_BN_CLICKED(IDC_RADIO_HDR, &CameraControl::OnBnClickedRadioHdr)
	ON_BN_CLICKED(IDC_RADIO_HIGHGAIN, &CameraControl::OnBnClickedRadioHighgain)
	ON_BN_CLICKED(IDC_RADIO_LOWGAIN, &CameraControl::OnBnClickedRadioLowgain)
	ON_WM_TIMER()
	ON_MESSAGE(TUMSG_FPS, OnTUMessageFps)
    ON_WM_CREATE()
    ON_WM_DESTROY()
    ON_BN_CLICKED(IDC_BTN_ONCEAE, &CameraControl::OnBnClickedBtnOnceae)
    ON_BN_CLICKED(RADIO_CMS, &CameraControl::OnBnClickedCms)
	ON_BN_CLICKED(IDC_BTN_USBLED, &CameraControl::OnBnClickedBtnUsbled)
	ON_BN_CLICKED(IDC_BTN_PIENABLE, &CameraControl::OnBnClickedBtnPienable)
	ON_CBN_SELCHANGE(IDC_CMB_BITOFDEPTH, &CameraControl::OnCbnSelchangeCmbBitofdepth)

	ON_BN_CLICKED(IDC_BTN_RATE, &CameraControl::OnBnClickedBtnRate)
	ON_CBN_SELCHANGE(IDC_CMB_TESTIMG, &CameraControl::OnCbnSelchangeCmbTestimg)
	ON_BN_CLICKED(IDC_BTN_DSNU, &CameraControl::OnBnClickedBtnDsnu)
	ON_BN_CLICKED(IDC_BTN_PRNU, &CameraControl::OnBnClickedBtnPrnu)
	ON_CBN_SELCHANGE(IDC_CMB_FAN, &CameraControl::OnCbnSelchangeCmbFan)
	ON_BN_CLICKED(IDC_BTN_TEC, &CameraControl::OnBnClickedBtnTec)
	ON_NOTIFY(NM_CUSTOMDRAW, IDC_SLD_COOLTARGET, &CameraControl::OnNMCustomdrawSldCooltarget)
	ON_CBN_SELCHANGE(IDC_CMB_GAIN, &CameraControl::OnCbnSelchangeCmbGain)
	ON_CBN_SELCHANGE(IDC_CMB_SHUTTER, &CameraControl::OnCbnSelchangeCmbShutter)
	ON_BN_CLICKED(IDC_CHK_TIMESTAMP, &CameraControl::OnBnClickedChkTimestamp)
	ON_CBN_SELCHANGE(IDC_CMB_IMGSELECT, &CameraControl::OnCbnSelchangeCmbImgselect)
	ON_NOTIFY(NM_CUSTOMDRAW, IDC_SLD_RATIO, &CameraControl::OnNMCustomdrawSldRatio)
END_MESSAGE_MAP()


// CameraControl message handlers

void CameraControl::InitControlRange()
{
	if (NULL != m_opCam.hIdxTUCam)
	{
		char szRes[64] = {0};
		TUCAM_CAPA_ATTR  attrCapa;
		TUCAM_PROP_ATTR  attrProp;
		TUCAM_VALUE_TEXT valText;

		valText.nTextSize = 64;
		valText.pText = &szRes[0];

		// ȡֱʷΧ
		attrCapa.idCapa = TUIDC_RESOLUTION;
		if (TUCAMRET_SUCCESS == TUCAM_Capa_GetAttr(m_opCam.hIdxTUCam, &attrCapa))
		{
			CString szRes;
			int nCnt = attrCapa.nValMax - attrCapa.nValMin + 1;
			valText.nID = TUIDC_RESOLUTION;
			m_cmbRes.ResetContent();
			for (int i=0; i<nCnt; ++i)
			{
				valText.dbValue = i;
				TUCAM_Capa_GetValueText(m_opCam.hIdxTUCam, &valText); 
				szRes = valText.pText;
				m_cmbRes.InsertString(i, szRes);
			}

			m_cmbRes.SetCurSel(0);
		}

		// ȡģʽλ
		attrCapa.idCapa = TUIDC_IMGMODESELECT;
		if (TUCAMRET_SUCCESS == TUCAM_Capa_GetAttr(m_opCam.hIdxTUCam, &attrCapa))
		{
			CString szMode;
			int nCnt = attrCapa.nValMax - attrCapa.nValMin + 1;
			m_nImgModeMin = attrCapa.nValMin;
			m_nImgModeMax = attrCapa.nValMax;
			valText.nID = TUIDC_IMGMODESELECT;
			m_cmbImgSelect.ResetContent();

			if (IsDhyana400BSIV3())
			{
				m_cmbImgSelect.InsertString(0, "CMS");
				m_cmbImgSelect.InsertString(1, "HDR");
				m_cmbImgSelect.InsertString(2, "HighSpeed");
				m_cmbImgSelect.InsertString(3, "GlobalReset");
			}
			else
			{
				for (int i = 0; i < nCnt; ++i)
				{
					valText.dbValue = i + m_nImgModeMin;
					TUCAM_Capa_GetValueText(m_opCam.hIdxTUCam, &valText);
					szMode = valText.pText;
					m_cmbImgSelect.InsertString(i, szMode);
				}
			}
			
			m_cmbImgSelect.SetCurSel(1);
			GetDlgItem(IDC_CMB_IMGSELECT)->ShowWindow(SW_SHOW);
			if(m_nImgModeMax < 2)
			{
				GetDlgItem(IDC_CMB_IMGSELECT)->ShowWindow(SW_HIDE);
				m_bSprtCMS = TRUE;
			}
			else
			{
				GetDlgItem(IDC_RADIO_CMS)->ShowWindow(SW_HIDE);
				GetDlgItem(IDC_CHK_BITOFDEPTH)->ShowWindow(SW_HIDE);
				m_bSprtCMS = FALSE;
			}
		}
		else
		{
			GetDlgItem(IDC_CMB_IMGSELECT)->ShowWindow(SW_HIDE);
		}

		// ȡλ
		attrCapa.idCapa = TUIDC_BITOFDEPTH;
		if (TUCAMRET_SUCCESS != TUCAM_Capa_GetAttr(m_opCam.hIdxTUCam, &attrCapa))
		{
			m_chkBDep.EnableWindow(FALSE);
			m_cmbBitOfDepth.ShowWindow(SW_HIDE);
		}
		else
		{
			CString szDepth;
			int nCnt = attrCapa.nValMax - attrCapa.nValMin + 1;

			if (attrCapa.nValMax == attrCapa.nValMin)
			{
				m_chkBDep.SetCheck((8 == attrCapa.nValMin) ? FALSE : TRUE);
				m_chkBDep.EnableWindow(FALSE);
				m_cmbBitOfDepth.ShowWindow(SW_HIDE);
			}
			else if (attrCapa.nValMin >= 8)
			{
				m_chkBDep.EnableWindow(TRUE);
				m_cmbBitOfDepth.ShowWindow(SW_HIDE);
			}
			else
			{
				m_chkBDep.ShowWindow(SW_HIDE);
				m_cmbBitOfDepth.ShowWindow(SW_SHOW);
				valText.nID = TUIDC_BITOFDEPTH;
				m_cmbBitOfDepth.ResetContent();

				for (int i = 0; i < nCnt; ++i)
				{
					valText.dbValue = i;
					TUCAM_Capa_GetValueText(m_opCam.hIdxTUCam, &valText);
					szDepth = valText.pText;
					m_cmbBitOfDepth.InsertString(i, szDepth);
				}
			}
		}

		// ȷΧ
		attrCapa.idCapa = TUIDC_FAN_GEAR;
		if (TUCAMRET_SUCCESS == TUCAM_Capa_GetAttr(m_opCam.hIdxTUCam, &attrCapa))
		{
			CString szFan;
			int nCnt = attrCapa.nValMax - attrCapa.nValMin + 1;
			valText.nID = TUIDC_FAN_GEAR;
			m_cmbFan.ResetContent();
			for (int i = 0; i<nCnt; ++i)
			{
				valText.dbValue = i;
				TUCAM_Capa_GetValueText(m_opCam.hIdxTUCam, &valText);
				szFan = valText.pText;
				m_cmbFan.InsertString(i, szFan);
			}

			if (attrCapa.nValMax < 0x05)
			{
				m_bSupportWaterCool = TRUE;
			}
			else
			{
				m_bSupportWaterCool = FALSE;
			}

			if (m_ccmode->IsDhyana95V2New() || m_ccmode->IsDhyana400BSIV3New())
			{
				m_bSupportWaterCool = FALSE;
			}

			m_cmbFan.SetCurSel(0);
		}
		else
		{
			GetDlgItem(IDC_STA_FAN)->EnableWindow(FALSE);
			GetDlgItem(IDC_CMB_FAN)->EnableWindow(FALSE);
		}

		// ȡԶع
		attrCapa.idCapa = TUIDC_ATEXPOSURE;
		if (TUCAMRET_SUCCESS != TUCAM_Capa_GetAttr(m_opCam.hIdxTUCam, &attrCapa))
		{
            GetDlgItem(IDC_BTN_ONCEAE)->ShowWindow(SW_HIDE);

            m_chkAE.ShowWindow(SW_SHOW);
            m_chkAE.EnableWindow(FALSE);
		}
		else
		{
            if (attrCapa.nValMax > 1)
            {
                GetDlgItem(IDC_BTN_ONCEAE)->ShowWindow(SW_SHOW);
                m_chkAE.ShowWindow(SW_HIDE);
            }
            else
            {
                GetDlgItem(IDC_BTN_ONCEAE)->ShowWindow(SW_HIDE);
                m_chkAE.ShowWindow(SW_SHOW);
                m_chkAE.EnableWindow(TRUE);
            }	
		}

		// ȡLED
		attrCapa.idCapa = TUIDC_ENABLELED;
		if (TUCAMRET_SUCCESS != TUCAM_Capa_GetAttr(m_opCam.hIdxTUCam, &attrCapa))
		{
			GetDlgItem(IDC_BTN_USBLED)->ShowWindow(SW_HIDE);
		}

		// ȡPIĤ
		attrCapa.idCapa = TUIDC_ENABLEPI;
		if (TUCAMRET_SUCCESS != TUCAM_Capa_GetAttr(m_opCam.hIdxTUCam, &attrCapa))
		{
			GetDlgItem(IDC_BTN_PIENABLE)->ShowWindow(SW_HIDE);
		}

		// ȡͼΧ
		attrCapa.idCapa = TUIDC_TESTIMGMODE;
		if (TUCAMRET_SUCCESS == TUCAM_Capa_GetAttr(m_opCam.hIdxTUCam, &attrCapa))
		{
			CString szTest;
			int nCnt = attrCapa.nValMax - attrCapa.nValMin + 1;
			valText.nID = TUIDC_TESTIMGMODE;
			m_cmbTestImg.ResetContent();
			for (int i=0; i<nCnt; ++i)
			{
				valText.dbValue = i;
				TUCAM_Capa_GetValueText(m_opCam.hIdxTUCam, &valText); 
				szTest = valText.pText;
				m_cmbTestImg.InsertString(i, szTest);
			}

			m_cmbTestImg.SetCurSel(0);
		}
		else
		{
            GetDlgItem(IDC_CMB_TESTIMG)->EnableWindow(FALSE);
		}

		// ȡPRNUʹ
		attrCapa.idCapa =  TUIDC_ENABLEIMGPRO;
		if (TUCAMRET_SUCCESS != TUCAM_Capa_GetAttr(m_opCam.hIdxTUCam, &attrCapa))
		{
			GetDlgItem(IDC_BTN_PRNU)->ShowWindow(SW_HIDE);
		}

		// ȡ俪״̬
		attrCapa.idCapa = TUIDC_ENABLETEC;
		if (TUCAMRET_SUCCESS != TUCAM_Capa_GetAttr(m_opCam.hIdxTUCam, &attrCapa))
		{
			///GetDlgItem(IDC_BTN_TEC)->ShowWindow(SW_HIDE);
			GetDlgItem(IDC_BTN_TEC)->EnableWindow(FALSE);
		}

		// ȡģʽ
		attrCapa.idCapa = TUIDC_SHUTTER;
		if (TUCAMRET_SUCCESS == TUCAM_Capa_GetAttr(m_opCam.hIdxTUCam, &attrCapa))
		{
			CString szShutter;
			int nCnt = attrCapa.nValMax - attrCapa.nValMin + 1;
			valText.nID = TUIDC_SHUTTER;

			m_cmbShutter.ResetContent();
			for (int i = 0; i<nCnt; ++i)
			{
				valText.dbValue = i;
				TUCAM_Capa_GetValueText(m_opCam.hIdxTUCam, &valText);
				szShutter = valText.pText;
				m_cmbShutter.InsertString(i, szShutter);
			}
		}
		else
		{
			GetDlgItem(IDC_STA_SHUTTER)->ShowWindow(SW_HIDE);
			GetDlgItem(IDC_CMB_SHUTTER)->ShowWindow(SW_HIDE);
		}

		// ȡԶعĿֵ
		attrProp.nIdxChn= 0;                    // ʹĬͨ
		attrProp.idProp = TUIDP_BRIGHTNESS;
		if (TUCAMRET_SUCCESS == TUCAM_Prop_GetAttr(m_opCam.hIdxTUCam, &attrProp))
		{
			m_bTargetRange = FALSE;
			m_sldTgr.SetRange((int)attrProp.dbValMin, (int)attrProp.dbValMax);
		}
		else
		{
			m_sldTgr.EnableWindow(FALSE);
			GetDlgItem(IDC_EDT_ATTGR)->EnableWindow(FALSE);
			GetDlgItem(IDC_STA_TARGET)->EnableWindow(FALSE);
		}

		attrProp.idProp = TUIDP_PIXELRATIO;
		if (TUCAMRET_SUCCESS == TUCAM_Prop_GetAttr(m_opCam.hIdxTUCam, &attrProp))
		{
			m_ccmode->m_bFixHeader = FALSE;
			m_bRatioRange = FALSE;
			m_sldRatio.SetRange((int)attrProp.dbValMin, (int)attrProp.dbValMax);
		}
		else
		{
			m_ccmode->m_bFixHeader = TRUE;
			m_sldRatio.EnableWindow(FALSE);
			GetDlgItem(IDC_EDT_RATIO)->EnableWindow(FALSE);
			GetDlgItem(IDC_STA_METERLEVEL)->EnableWindow(FALSE);
		}

		attrProp.idProp = TUIDP_FRAME_RATE;
		if (TUCAMRET_SUCCESS != TUCAM_Prop_GetAttr(m_opCam.hIdxTUCam, &attrProp))
		{
			GetDlgItem(IDC_STA_RATE)->EnableWindow(FALSE);
			GetDlgItem(IDC_EDT_RATE)->EnableWindow(FALSE);
			GetDlgItem(IDC_BTN_RATE)->EnableWindow(FALSE);
		}

        // ȡ淶Χ
		attrProp.idProp = TUIDP_GLOBALGAIN;
		if (TUCAMRET_SUCCESS == TUCAM_Prop_GetAttr(m_opCam.hIdxTUCam, &attrProp))
		{
			m_bGainRange = FALSE;
            m_uiMaxGain  = (UINT)attrProp.dbValMax;

            if (m_uiMaxGain > 6)
            {
                m_sldGain.SetRange((int)attrProp.dbValMin, (int)attrProp.dbValMax);

                GetDlgItem(IDC_STA_GAIN)->ShowWindow(SW_SHOW);
                GetDlgItem(IDC_SLD_GAIN)->ShowWindow(SW_SHOW);
                GetDlgItem(IDC_EDT_GAIN)->ShowWindow(SW_SHOW);
				GetDlgItem(IDC_CMB_GAIN)->ShowWindow(SW_HIDE);
                GetDlgItem(IDC_RADIO_HDR)->ShowWindow(SW_HIDE);
                GetDlgItem(IDC_RADIO_HIGHGAIN)->ShowWindow(SW_HIDE);
                GetDlgItem(IDC_RADIO_LOWGAIN)->ShowWindow(SW_HIDE);
                GetDlgItem(IDC_RADIO_CMS)->ShowWindow(SW_HIDE);

                GetDlgItem(IDC_STA_GAIN)->EnableWindow(TRUE);
                GetDlgItem(IDC_SLD_GAIN)->EnableWindow(TRUE);
                GetDlgItem(IDC_EDT_GAIN)->EnableWindow(TRUE);  
				GetDlgItem(IDC_BTN_DSNU)->ShowWindow(SW_HIDE);
		
            }
            else
            {
                GetDlgItem(IDC_STA_GAIN)->ShowWindow(SW_HIDE);
                GetDlgItem(IDC_SLD_GAIN)->ShowWindow(SW_HIDE);
                GetDlgItem(IDC_EDT_GAIN)->ShowWindow(SW_HIDE);
                GetDlgItem(IDC_RADIO_HDR)->ShowWindow(SW_SHOW);
                GetDlgItem(IDC_RADIO_HIGHGAIN)->ShowWindow(SW_SHOW);
                GetDlgItem(IDC_RADIO_LOWGAIN)->ShowWindow(SW_SHOW);

                GetDlgItem(IDC_RADIO_HDR)->EnableWindow(TRUE);
                GetDlgItem(IDC_RADIO_HIGHGAIN)->EnableWindow(TRUE);
                GetDlgItem(IDC_RADIO_LOWGAIN)->EnableWindow(TRUE);
				GetDlgItem(IDC_CMB_GAIN)->ShowWindow(SW_HIDE);

				if ((int)attrProp.dbValMin > 0)
				{
					GetDlgItem(IDC_RADIO_HDR)->EnableWindow(FALSE);
				}

				if (DHYANA_1605BSI_PID == m_nPID || DHYANA_ULTRA1605_PID == m_nPID)
				{
					CString szTest;
					int nCnt = (int)(attrProp.dbValMax - attrProp.dbValMin + 1);
					valText.nID = TUIDP_GLOBALGAIN;
					m_cmbGain.ResetContent();

					for (int i = 0; i<nCnt; ++i)
					{
						valText.dbValue = i;
						TUCAM_Prop_GetValueText(m_opCam.hIdxTUCam, &valText);
						szTest = valText.pText;
						m_cmbGain.InsertString(i, szTest);
					}

					m_cmbGain.SetCurSel(0);

					GetDlgItem(IDC_STA_GAIN)->ShowWindow(SW_SHOW);
					GetDlgItem(IDC_CMB_GAIN)->ShowWindow(SW_SHOW);
					GetDlgItem(IDC_RADIO_HDR)->ShowWindow(SW_HIDE);
					GetDlgItem(IDC_RADIO_HIGHGAIN)->ShowWindow(SW_HIDE);
					GetDlgItem(IDC_RADIO_LOWGAIN)->ShowWindow(SW_HIDE);
				}
            }
		} 
		else
		{
            GetDlgItem(IDC_STA_GAIN)->ShowWindow(SW_SHOW);
            GetDlgItem(IDC_SLD_GAIN)->ShowWindow(SW_SHOW);
            GetDlgItem(IDC_EDT_GAIN)->ShowWindow(SW_SHOW);
            GetDlgItem(IDC_RADIO_HDR)->ShowWindow(SW_HIDE);
            GetDlgItem(IDC_RADIO_HIGHGAIN)->ShowWindow(SW_HIDE);
            GetDlgItem(IDC_RADIO_LOWGAIN)->ShowWindow(SW_HIDE);

            GetDlgItem(IDC_STA_GAIN)->EnableWindow(FALSE);
			GetDlgItem(IDC_SLD_GAIN)->EnableWindow(FALSE);
			GetDlgItem(IDC_EDT_GAIN)->EnableWindow(FALSE);
		}

		// ȡعʱ䷶Χ
		attrProp.idProp = TUIDP_EXPOSURETM;
		if (TUCAMRET_SUCCESS == TUCAM_Prop_GetAttr(m_opCam.hIdxTUCam, &attrProp))
		{
			((CButton*)GetDlgItem(IDC_SPN_SEC))->EnableWindow(TRUE);
			((CButton*)GetDlgItem(IDC_EDT_SEC))->EnableWindow(TRUE);
			((CButton*)GetDlgItem(IDC_SPN_MS))->EnableWindow(TRUE);
			((CButton*)GetDlgItem(IDC_EDT_MS))->EnableWindow(TRUE);
	
			if (m_nPID == 0xEC07 || m_nPID == 0xEC09)
			{
				attrProp.dbValMin = 0.13;
				attrProp.dbValMax = 15000000;
			}
			else if (m_nPID == 0xE407)
			{
				attrProp.dbValMax = 14000000;
			}
			else if (m_nPID == 0xE40A)
			{
				attrProp.dbValMax = 12000000;
			}
			

			m_uiExpStep = (UINT)(attrProp.dbValStep * 1000 + 0.5f);

			if((DWORD)(attrProp.dbValMax * 1000) < 1000000)
			{
				((CButton*)GetDlgItem(IDC_SPN_SEC))->EnableWindow(FALSE);
				((CButton*)GetDlgItem(IDC_EDT_SEC))->EnableWindow(FALSE);
			}

			if((DWORD)(attrProp.dbValMax * 1000) < 1000)
			{
				((CButton*)GetDlgItem(IDC_SPN_SEC))->EnableWindow(FALSE);
				((CButton*)GetDlgItem(IDC_EDT_SEC))->EnableWindow(FALSE);
				((CButton*)GetDlgItem(IDC_SPN_MS))->EnableWindow(FALSE);
				((CButton*)GetDlgItem(IDC_EDT_MS))->EnableWindow(FALSE);
			}
		} 

		// ȡ¶ȷΧ
		attrProp.idProp = TUIDP_TEMPERATURE;
		if (TUCAMRET_SUCCESS == TUCAM_Prop_GetAttr(m_opCam.hIdxTUCam, &attrProp))
		{
			TUCAM_VALUE_INFO valInfo;
			valInfo.nID = TUIDI_ZEROTEMPERATURE_VALUE;
			TUCAM_Dev_GetInfo(m_opCam.hIdxTUCam, &valInfo);

			m_nMidTTemp = valInfo.nValue;
			m_sldCoolT.SetRange((int)attrProp.dbValMin, (int)attrProp.dbValMax);
		}
		else
		{
			GetDlgItem(IDC_STA_COOLTARGET)->EnableWindow(FALSE);
			GetDlgItem(IDC_SLD_COOLTARGET)->EnableWindow(FALSE);
			GetDlgItem(IDC_EDT_COOLTARGET)->EnableWindow(FALSE);
		}
	}
}

void CameraControl::RefreshValue()
{
	if (NULL != m_opCam.hIdxTUCam)
	{
	    // 
        int     nVal  = 0;
        double  dbVal = 0;
        CString szText= _T("");

        // ȡǰֱֵ
        if (TUCAMRET_SUCCESS == TUCAM_Capa_GetValue(m_opCam.hIdxTUCam, TUIDC_RESOLUTION, &nVal))
        {
            m_cmbRes.SetCurSel(nVal);
        }

        // ȡǰλ
        if (TUCAMRET_SUCCESS == TUCAM_Capa_GetValue(m_opCam.hIdxTUCam, TUIDC_BITOFDEPTH, &nVal))
        {
            if (nVal == 8)
            {
                m_chkBDep.SetCheck(0);
            }
            else
            {
                m_chkBDep.SetCheck(1);
            }

			if (m_cmbBitOfDepth.IsWindowVisible())
				m_cmbBitOfDepth.SetCurSel(nVal);
        }

		// ȡֵ
		if (TUCAMRET_SUCCESS == TUCAM_Capa_GetValue(m_opCam.hIdxTUCam, TUIDC_FAN_GEAR, &nVal))
		{
			m_cmbFan.SetCurSel(nVal);
		}

		// ȡͼֵ
		if (TUCAMRET_SUCCESS == TUCAM_Capa_GetValue(m_opCam.hIdxTUCam, TUIDC_TESTIMGMODE, &nVal))
		{
			m_cmbTestImg.SetCurSel(nVal);
		}

		// ȡǰ״̬
		if (TUCAMRET_SUCCESS == TUCAM_Capa_GetValue(m_opCam.hIdxTUCam, TUIDC_ENABLETEC, &nVal))
		{
			m_bTEC = nVal;
			TECShow(m_bTEC);
		}

		// ȡģʽ
		if (TUCAMRET_SUCCESS == TUCAM_Capa_GetValue(m_opCam.hIdxTUCam, TUIDC_SHUTTER, &nVal))
		{
			m_cmbShutter.SetCurSel(nVal);
		}

		// ʱ
		if (TUCAMRET_SUCCESS == TUCAM_Capa_GetValue(m_opCam.hIdxTUCam, TUIDC_ENABLETIMESTAMP, &nVal))
		{
			m_ccmode->m_bTimeStamp = nVal;
			m_chkTimestamp.SetCheck(nVal);
			GetDlgItem(IDC_BTN_ONCEAE)->EnableWindow((bool)nVal);
		}
		else
		{
			((CButton*)GetDlgItem(IDC_CHK_TIMESTAMP))->ShowWindow(SW_HIDE);
		}

		// ȡǰعʱ
        if (TUCAMRET_SUCCESS == TUCAM_Prop_GetValue(m_opCam.hIdxTUCam, TUIDP_EXPOSURETM, &dbVal))
        {
            UpdateExposureTime((DWORD)(dbVal * 1000));
        }

        // ȡǰĿֵ
        if (TUCAMRET_SUCCESS == TUCAM_Prop_GetValue(m_opCam.hIdxTUCam, TUIDP_BRIGHTNESS, &dbVal))
        {    
            nVal = (int)dbVal;
            szText.Format(_T("%d%%"), nVal);
            GetDlgItem(IDC_EDT_ATTGR)->SetWindowText(szText);    
			m_bTargetRange = FALSE;
            m_sldTgr.SetPos(nVal);
			m_bTargetRange = TRUE;
        }

		// ȡǰPixelRatio
		if (TUCAMRET_SUCCESS == TUCAM_Prop_GetValue(m_opCam.hIdxTUCam, TUIDP_PIXELRATIO, &dbVal))
		{
			nVal = (int)dbVal;
			szText.Format(_T("%0.2f%%"), dbVal/100.0);
			GetDlgItem(IDC_EDT_RATIO)->SetWindowText(szText);
			m_bRatioRange = FALSE;
			m_sldRatio.SetPos(nVal);
			m_bRatioRange = TRUE;
		}

		// ȡǰֵ֡
		if (TUCAMRET_SUCCESS == TUCAM_Prop_GetValue(m_opCam.hIdxTUCam, TUIDP_FRAME_RATE, &dbVal))
		{    
			szText.Format(_T("%.2f"), dbVal);
			GetDlgItem(IDC_EDT_RATE)->SetWindowText(szText);    
		}

		// ȡǰģʽ
		int nImgMode = 0;
		if (TUCAMRET_SUCCESS == TUCAM_Capa_GetValue(m_opCam.hIdxTUCam, TUIDC_IMGMODESELECT, &nVal))
		{    
			if (IsDhyana400BSIV3())
			{
				if (0x01 == nVal)                      // CMS Mode
				{
					nImgMode = TUIM_CMS;
				}
				else if (0x02 == nVal)                 // HDR Mode
				{
					nImgMode = TUIM_HDR;
				}
				else if (0x03 == nVal || 0x04 == nVal) // HighSpeed High/Low
				{
					nImgMode = TUIM_HIGHSPEED;
				}
				else if (0x05 == nVal)                 // GlobalReset
				{
					nImgMode = TUIM_GLOBAL_RESET;
				}
			}
			else
			{
				nImgMode = nVal;
			}
			m_ccmode->m_nImgMode = nImgMode;
		
			m_cmbImgSelect.SetCurSel((nImgMode - m_nImgModeMin));
			GainModeEnable();
		}
		else
		{
			GetDlgItem(IDC_RADIO_CMS)->ShowWindow(SW_HIDE);
		}

        // ȡԶع״̬
        if (TUCAMRET_SUCCESS == TUCAM_Capa_GetValue(m_opCam.hIdxTUCam, TUIDC_ATEXPOSURE, &nVal))
        {
            // Զع
            if (1 == nVal)
            {
                m_chkAE.SetCheck(TRUE);

                m_sldTgr.EnableWindow(TRUE);
                GetDlgItem(IDC_EDT_ATTGR)->EnableWindow(TRUE);

                GetDlgItem(IDC_STA_SEC)->EnableWindow(FALSE);
                GetDlgItem(IDC_STA_MS)->EnableWindow(FALSE);
                GetDlgItem(IDC_STA_US)->EnableWindow(FALSE);

                GetDlgItem(IDC_EDT_SEC)->EnableWindow(FALSE);
                GetDlgItem(IDC_EDT_MS)->EnableWindow(FALSE);
                GetDlgItem(IDC_EDT_US)->EnableWindow(FALSE);
                GetDlgItem(IDC_SPN_SEC)->EnableWindow(FALSE);
                GetDlgItem(IDC_SPN_MS)->EnableWindow(FALSE);
                GetDlgItem(IDC_SPN_US)->EnableWindow(FALSE);

                GetDlgItem(IDC_BTN_EXPTM)->EnableWindow(FALSE);
				
				SetTimer(IDT_GET_AE, IDT_SPC_AE, NULL);	

				if (m_nPID == FL20BW_PID_SSC183IM1)
				{
					GetDlgItem(IDC_STA_GAIN)->EnableWindow(FALSE);
					GetDlgItem(IDC_SLD_GAIN)->EnableWindow(FALSE);
					GetDlgItem(IDC_EDT_GAIN)->EnableWindow(FALSE);
				}
            }
            else
            {
                m_chkAE.SetCheck(FALSE);

                ///m_sldTgr.EnableWindow(FALSE);
                ///GetDlgItem(IDC_EDT_ATTGR)->EnableWindow(FALSE);

                GetDlgItem(IDC_STA_SEC)->EnableWindow(TRUE);
                GetDlgItem(IDC_STA_MS)->EnableWindow(TRUE);
                GetDlgItem(IDC_STA_US)->EnableWindow(TRUE);

                GetDlgItem(IDC_EDT_SEC)->EnableWindow(TRUE);
                GetDlgItem(IDC_EDT_MS)->EnableWindow(TRUE);
                GetDlgItem(IDC_EDT_US)->EnableWindow(TRUE);
                GetDlgItem(IDC_SPN_SEC)->EnableWindow(TRUE);
                GetDlgItem(IDC_SPN_MS)->EnableWindow(TRUE);
                GetDlgItem(IDC_SPN_US)->EnableWindow(TRUE);

                GetDlgItem(IDC_BTN_EXPTM)->EnableWindow(TRUE);
            }
        }

		// ȡǰֵ
		if (TUCAMRET_SUCCESS == TUCAM_Prop_GetValue(m_opCam.hIdxTUCam, TUIDP_GLOBALGAIN, &dbVal))
		{
            m_nGain = (int)dbVal;

            if (m_uiMaxGain > 6)
            {
                m_sldGain.SetPos(m_nGain);

                szText.Format(_T("%d"), m_nGain);
                GetDlgItem(IDC_EDT_GAIN)->SetWindowText(szText);  
            }
			else if (DHYANA_1605BSI_PID == m_nPID || DHYANA_ULTRA1605_PID == m_nPID)
			{
				m_cmbGain.SetCurSel(m_nGain);

				if (TUCAMRET_SUCCESS == TUCAM_Capa_GetValue(m_opCam.hIdxTUCam, TUIDC_ENABLEDSNU, &nVal))
				{
					m_bDSNUState = nVal;
				}

				DSNUEnableShow(m_bDSNUState);
			}
            else
            {
				if ( m_nGain >  0x02)
				{
					m_nGain =  m_nGain - 0x03;
					m_bDSNUState = FALSE;
				}
				else
				{
					m_bDSNUState = TRUE;
				}

                DSNUEnableShow(m_bDSNUState);
				GainModeShow();

				if (m_bSprtCMS)
				{
					if (TUCAMRET_SUCCESS == TUCAM_Capa_GetValue(m_opCam.hIdxTUCam, TUIDC_IMGMODESELECT, &nVal))
					{
						 if (1 == nVal)
						 {
							 ((CButton *)GetDlgItem(IDC_RADIO_HDR))->SetCheck(FALSE);
							 ((CButton *)GetDlgItem(IDC_RADIO_HIGHGAIN))->SetCheck(FALSE);
							 ((CButton *)GetDlgItem(IDC_RADIO_LOWGAIN))->SetCheck(FALSE);
							 ((CButton *)GetDlgItem(IDC_RADIO_CMS))->SetCheck(TRUE);
						 }
					}
				}
            }
		}

		// ȡǰLed ״̬
		if (TUCAMRET_SUCCESS == TUCAM_Capa_GetValue(m_opCam.hIdxTUCam, TUIDC_ENABLELED, &nVal))
		{
			m_bLedState  = (BOOL) nVal;
			LedShow(m_bLedState);
		}

		// ȡǰPIĤ״̬
		if (TUCAMRET_SUCCESS == TUCAM_Capa_GetValue(m_opCam.hIdxTUCam, TUIDC_ENABLEPI, &nVal))
		{
			m_bPIState  = (BOOL) nVal;
			PIEnableShow(m_bPIState);
		}

		// ȡǰPRNU״̬
		if (TUCAMRET_SUCCESS == TUCAM_Capa_GetValue(m_opCam.hIdxTUCam, TUIDC_ENABLEIMGPRO, &nVal))
		{
			m_bPRNUState = (0x04 & nVal) ? TRUE : FALSE;
			PRNUEnableShow(m_bPRNUState);
		}

        // ȡǰĿ¶
        if (TUCAMRET_SUCCESS == TUCAM_Prop_GetValue(m_opCam.hIdxTUCam, TUIDP_TEMPERATURE_TARGET, &dbVal))
        {
            m_bTempChange = FALSE;
            m_sldCoolT.SetPos((int)dbVal);
            m_bTempChange = TRUE;

            CString szTemp = _T("0 ");
			if (m_nMidTTemp > 100)
			{
				szTemp.Format(_T("%.1f "), (dbVal - m_nMidTTemp) / 10.0);
			}
			else
			{
				szTemp.Format(_T("%d "), ((int)dbVal - m_nMidTTemp));
			}
           
            GetDlgItem(IDC_EDT_COOLTARGET)->SetWindowText(szTemp);
        }
	}
}

void CameraControl::RefreshBtn(BOOL bLiving)
{
	GetDlgItem(IDC_BTN_LIVE)->EnableWindow(bLiving);
}

void CameraControl::RefreshOpen(TUCAM_OPEN &opCam)             // ˢ
{
	m_opCam  = opCam;
}

void CameraControl::BtnShow(BOOL mLiving)
{
	if(mLiving)
    {
		GetDlgItem(IDC_BTN_LIVE)->SetWindowText(_T("Stop"));
        GetDlgItem(IDC_CMB_RES)->EnableWindow(FALSE);
		GetDlgItem(IDC_CHK_TIMESTAMP)->EnableWindow(FALSE);
    }
	else
    {
		GetDlgItem(IDC_BTN_LIVE)->SetWindowText(_T("Live"));
        GetDlgItem(IDC_CMB_RES)->EnableWindow(TRUE);
		GetDlgItem(IDC_CHK_TIMESTAMP)->EnableWindow(TRUE);
    }
}

void CameraControl::LedShow(BOOL bLed)
{
	if(bLed)
	{
		GetDlgItem(IDC_BTN_USBLED)->SetWindowText(_T("Led On"));
	}
	else
	{
		GetDlgItem(IDC_BTN_USBLED)->SetWindowText(_T("Led Off"));
	}
}

void CameraControl::PIEnableShow(BOOL bPI)
{
	if(bPI)
	{
		GetDlgItem(IDC_BTN_PIENABLE)->SetWindowText(_T("PI On"));
	}
	else
	{
		GetDlgItem(IDC_BTN_PIENABLE)->SetWindowText(_T("PI Off"));
	}
}

void CameraControl::DSNUEnableShow(BOOL bDSNU)  
{
	if(bDSNU)
	{
		GetDlgItem(IDC_BTN_DSNU)->SetWindowText(_T("DSNU On"));
	}
	else
	{
		GetDlgItem(IDC_BTN_DSNU)->SetWindowText(_T("DSNU Off"));
	}
}

void CameraControl::PRNUEnableShow(BOOL bPRNU)
{
	if(bPRNU)
	{
		GetDlgItem(IDC_BTN_PRNU)->SetWindowText(_T("PRNU On"));
	}
	else
	{
		GetDlgItem(IDC_BTN_PRNU)->SetWindowText(_T("PRNU Off"));
	}
}

void CameraControl::SetGain()
{
	int nVal = 0;
	int nGain = 0;
	int nImgMode = m_ccmode->m_nImgMode;
	double dbVal = 0;
	if (NULL != m_opCam.hIdxTUCam)
	{
		if (IsDhyana400BSIV3())
		{
			switch (m_ccmode->m_nImgMode)
			{
			case TUIM_CMS:
				nImgMode = 0x01;
				m_nGain  = 0x00;
				break;
			case TUIM_HDR:
				nImgMode = 0x02;
				break;
			case TUIM_HIGHSPEED:
				m_nGain = (0x00 == m_nGain) ? 0x01 : m_nGain;
				nImgMode = (0x01 == m_nGain) ? 0x03 : 0x04;
				break;
			case TUIM_GLOBAL_RESET:
				nImgMode = 0x05;
				m_nGain = (0x00 == m_nGain) ? 0x01 : m_nGain;
				break;
			default:
				break;
			}
		}

		if (TUCAMRET_SUCCESS == TUCAM_Capa_GetValue(m_opCam.hIdxTUCam, TUIDC_IMGMODESELECT, &nVal))
		{
			if (nVal != nImgMode)
			{
				TUCAM_Capa_SetValue(m_opCam.hIdxTUCam, TUIDC_IMGMODESELECT, nImgMode);
			}
		}

		nGain = m_bDSNUState ? m_nGain : (m_nGain + 3);
		TUCAM_Prop_SetValue(m_opCam.hIdxTUCam, TUIDP_GLOBALGAIN, nGain);
		if (IsDhyana400BSIV3() && m_ccmode->m_nImgMode == TUIM_HIGHSPEED)
		{
			RefreshRate();
		}
		
	}
}

void  CameraControl::GainModeEnable()
{
	if (m_nImgModeMax < TUIM_HDR)
	{
		return;
	}

	int nCur  = m_ccmode->m_nImgMode;
	int nGain = m_nGain;

	if (TUIM_CMS == m_nImgModeMin)
	{
		switch (nCur)
		{
		case TUIM_CMS:
			if (0 != nGain)
			{
				m_nGain = 0;
			}
			GetDlgItem(IDC_RADIO_HDR)->EnableWindow(FALSE);
			GetDlgItem(IDC_RADIO_HIGHGAIN)->EnableWindow(FALSE);
			GetDlgItem(IDC_RADIO_LOWGAIN)->EnableWindow(FALSE);

			break;
		case TUIM_HDR:
			GetDlgItem(IDC_RADIO_HDR)->EnableWindow(TRUE);
			GetDlgItem(IDC_RADIO_HIGHGAIN)->EnableWindow(TRUE);
			GetDlgItem(IDC_RADIO_LOWGAIN)->EnableWindow(TRUE);
			break;
		case TUIM_HIGHSPEED:
		case TUIM_GLOBAL_RESET:
			if (0 == nGain)
			{
				m_nGain = 1;
			}
			GetDlgItem(IDC_RADIO_HDR)->EnableWindow(FALSE);
			GetDlgItem(IDC_RADIO_HIGHGAIN)->EnableWindow(TRUE);
			GetDlgItem(IDC_RADIO_LOWGAIN)->EnableWindow(TRUE);

			break;
		default:
			break;
		}
	}
	else
	{
		GetDlgItem(IDC_RADIO_HDR)->EnableWindow(0 == nCur);
		GetDlgItem(IDC_RADIO_HIGHGAIN)->EnableWindow(0 == nCur);
		GetDlgItem(IDC_RADIO_LOWGAIN)->EnableWindow(0 == nCur);
	}
}

void CameraControl::GainModeShow()
{
	switch (m_nGain)
	{
	case 1:
	{
		((CButton *)GetDlgItem(IDC_RADIO_HDR))->SetCheck(FALSE);
		((CButton *)GetDlgItem(IDC_RADIO_HIGHGAIN))->SetCheck(TRUE);
		((CButton *)GetDlgItem(IDC_RADIO_LOWGAIN))->SetCheck(FALSE);
		((CButton *)GetDlgItem(IDC_RADIO_CMS))->SetCheck(FALSE);
	}
	break;
	case 2:
	{
		((CButton *)GetDlgItem(IDC_RADIO_HDR))->SetCheck(FALSE);
		((CButton *)GetDlgItem(IDC_RADIO_HIGHGAIN))->SetCheck(FALSE);
		((CButton *)GetDlgItem(IDC_RADIO_LOWGAIN))->SetCheck(TRUE);
		((CButton *)GetDlgItem(IDC_RADIO_CMS))->SetCheck(FALSE);
	}
	break;
	case 0:
	default:
	{
		((CButton *)GetDlgItem(IDC_RADIO_HDR))->SetCheck(TRUE);
		((CButton *)GetDlgItem(IDC_RADIO_HIGHGAIN))->SetCheck(FALSE);
		((CButton *)GetDlgItem(IDC_RADIO_LOWGAIN))->SetCheck(FALSE);
		((CButton *)GetDlgItem(IDC_RADIO_CMS))->SetCheck(FALSE);
	}
	break;
	}
}

void CameraControl::OnTimer(UINT_PTR nIDEvent)
{
    // TODO: ڴϢ/Ĭֵ
	if (IDT_GET_FPS_RATE == nIDEvent) 
	{
		CString szFps;
		szFps.Format(_T("%.2f fps"), m_ccmode->m_fFps / 1000.0f);

		GetDlgItem(IDC_STA_FPS)->SetWindowText(szFps); 
	}
    else if (IDT_GET_TEMPERATURE == nIDEvent) 
    {
		int nVal = 0;
        double dblVal = 1.0f;

        if (NULL != m_opCam.hIdxTUCam)
        {
            if (TUCAMRET_SUCCESS == TUCAM_Prop_GetValue(m_opCam.hIdxTUCam, TUIDP_TEMPERATURE, &dblVal))
            {
                CString sz;
                sz.Format(_T("%.2f "), dblVal);
                GetDlgItem(IDC_STA_TEMP)->SetWindowText(sz);

				if (m_bSupportWaterCool)
				{
					// ȡֵ
					if (TUCAMRET_SUCCESS == TUCAM_Capa_GetValue(m_opCam.hIdxTUCam, TUIDC_FAN_GEAR, &nVal))
					{
						if (0x03 == nVal && dblVal > 0)
						{
							TUCAM_Capa_SetValue(m_opCam.hIdxTUCam, TUIDC_FAN_GEAR, 0);
							m_cmbFan.SetCurSel(0);
						}
					}
				}
            }

			TUCAM_VALUE_INFO valInfo;

			// ȡ¶ȱ
			valInfo.nID = TUIDI_TEMPALARMFLAG;

			if (TUCAMRET_SUCCESS == TUCAM_Dev_GetInfo(m_opCam.hIdxTUCam, &valInfo))
			{
				// FPGA±
				if (1 == valInfo.nValue)
				{
					if (!m_bAlarmState && !m_bMessageBoxShow)
					{
						m_bAlarmState = true;
						m_bMessageBoxShow = true;
						AfxMessageBox(_T("Camera temperature is too high. Please check the environment around the camera and shut down the camera as soon as possible!"));
						m_bMessageBoxShow = false;
					}
				}
				else
				{
					m_bAlarmState = false;
				}
			}
        }
    }
	else if (IDT_GET_AE == nIDEvent)
	{
		if (NULL != m_opCam.hIdxTUCam)
		{
			double dbVal = 0;

			if (m_uiMaxGain > 6)
			{
				// ȡǰֵ
				if (TUCAMRET_SUCCESS == TUCAM_Prop_GetValue(m_opCam.hIdxTUCam, TUIDP_GLOBALGAIN, &dbVal))
				{
					m_nGain = (int)dbVal;
					m_bGainRange = FALSE;
					m_sldGain.SetPos(m_nGain);

					CString szText;
					szText.Format(_T("%d"), m_nGain);
					GetDlgItem(IDC_EDT_GAIN)->SetWindowText(szText);  
				}
			}

			// ȡǰعʱ
			if (TUCAMRET_SUCCESS == TUCAM_Prop_GetValue(m_opCam.hIdxTUCam, TUIDP_EXPOSURETM, &dbVal))
			{
				UpdateExposureTime((DWORD)(dbVal * 1000));
			}
		}
	}
    else if (IDT_GET_ONCE_AE == nIDEvent)
    {
        if (NULL != m_opCam.hIdxTUCam)
        {
            int nVal = 2;
            if (TUCAMRET_SUCCESS == TUCAM_Capa_GetValue(m_opCam.hIdxTUCam, TUIDC_ATEXPOSURE, &nVal))
            {
                if (0 == nVal)
                {
                    GetDlgItem(IDC_BTN_ONCEAE)->EnableWindow(TRUE);
                    KillTimer(IDT_GET_ONCE_AE);

					GetDlgItem(IDC_EDT_ATTGR)->EnableWindow(TRUE);

                    GetDlgItem(IDC_STA_SEC)->EnableWindow(TRUE);
                    GetDlgItem(IDC_STA_MS)->EnableWindow(TRUE);
                    GetDlgItem(IDC_STA_US)->EnableWindow(TRUE);

                    GetDlgItem(IDC_EDT_SEC)->EnableWindow(TRUE);
                    GetDlgItem(IDC_EDT_MS)->EnableWindow(TRUE);
                    GetDlgItem(IDC_EDT_US)->EnableWindow(TRUE);
                    GetDlgItem(IDC_SPN_SEC)->EnableWindow(TRUE);
                    GetDlgItem(IDC_SPN_MS)->EnableWindow(TRUE);
                    GetDlgItem(IDC_SPN_US)->EnableWindow(TRUE);

                    GetDlgItem(IDC_BTN_EXPTM)->EnableWindow(TRUE);

                    double dbVal = 0.0f;
                    // ȡǰعʱ
                    if (TUCAMRET_SUCCESS == TUCAM_Prop_GetValue(m_opCam.hIdxTUCam, TUIDP_EXPOSURETM, &dbVal))
                    {
                        UpdateExposureTime((DWORD)(dbVal * 1000));
                    }
                }
            }
        }
    }
   
    CDialog::OnTimer(nIDEvent);
}

LRESULT CameraControl::OnTUMessageFps(WPARAM wParam, LPARAM lParam)
{   
    int nFps = (int)lParam;
	float m_dfps = nFps / 1000.0f;
    CString szFps;
    szFps.Format(_T("%.2f fps"), m_dfps);
    GetDlgItem(IDC_STA_FPS)->SetWindowText(szFps);

    return 0;
}

BOOL CameraControl::PreTranslateMessage(MSG* pMsg)
{
    // TODO: ڴרô/û
    if((pMsg->wParam == VK_ESCAPE))
    {  
        return TRUE;  
    }

    if ((pMsg->wParam == VK_RETURN))
    {
        //UpdateData();
        return TRUE;
    }

	if(TranslateAccelerator(m_hWnd,m_ccmode->hAccKey,pMsg))  
	{    
		return TRUE;    
	}  

    return CDialog::PreTranslateMessage(pMsg);
}

void CameraControl::OnBnClickedBtnLive()
{
	// TODO: Add your control notification handler code here
    /*if (m_ccmode->m_bWaitting)
    {
//      SaveProfile();
        if(NULL != m_ccmode->m_tabPage[4])
        {
            m_ccmode->m_nTriMode = TUCCM_SEQUENCE;
            ((Trigger*)m_ccmode->m_tabPage[4])->EnableTriggerControl(FALSE);

            ((CButton *)m_ccmode->m_tabPage[4]->GetDlgItem(IDC_RADIO_OFF))->SetCheck(TRUE);
            ((CButton *)m_ccmode->m_tabPage[4]->GetDlgItem(IDC_RADIO_STANDARD))->SetCheck(FALSE);
            ((CButton *)m_ccmode->m_tabPage[4]->GetDlgItem(IDC_RADIO_SYNCHRO))->SetCheck(FALSE);
            ((CButton *)m_ccmode->m_tabPage[4]->GetDlgItem(IDC_RADIO_GLOABAL))->SetCheck(FALSE);
        }
    }
    else
    {
        m_ccmode->m_nTriMode = TUCCM_SEQUENCE;
    }*/
    
	m_ccmode->OnBnClickedBtnLive();

}

void CameraControl::OnCbnSelchangeCmbRes()
{
	// TODO: Add your control notification handler code here
	if (NULL != m_opCam.hIdxTUCam)
	{
        BOOL bWaitting = m_ccmode->m_bWaitting;

        if (bWaitting)
        {
            m_ccmode->StopWaitForFrame();
        }


		TUCAM_Capa_SetValue(m_opCam.hIdxTUCam, TUIDC_RESOLUTION, m_cmbRes.GetCurSel());
        RefreshRate();
        if (NULL != ((ROI*)(m_ccmode->m_tabPage[3])))
        {
            ((ROI*)(m_ccmode->m_tabPage[3]))->CloseROIState();
        }
		m_ccmode->OnRefreshLevelRange();
		OnBnClickedBtnExptm();

        m_ccmode->UpdateWindow();

        if (bWaitting)
        {
            m_ccmode->StartWaitForFrame();
        }
	}
}

void CameraControl::OnBnClickedChkAe()
{
	// TODO: Add your control notification handler code here
	double dbVal = 0;     

    if (m_chkAE.GetCheck())
    {
        m_sldTgr.EnableWindow(TRUE);        
        TUCAM_Capa_SetValue(m_opCam.hIdxTUCam, TUIDC_ATEXPOSURE, 1);

        GetDlgItem(IDC_EDT_ATTGR)->EnableWindow(TRUE);

        GetDlgItem(IDC_STA_SEC)->EnableWindow(FALSE);
        GetDlgItem(IDC_STA_MS)->EnableWindow(FALSE);
        GetDlgItem(IDC_STA_US)->EnableWindow(FALSE);

        GetDlgItem(IDC_EDT_SEC)->EnableWindow(FALSE);
        GetDlgItem(IDC_EDT_MS)->EnableWindow(FALSE);
        GetDlgItem(IDC_EDT_US)->EnableWindow(FALSE);
        GetDlgItem(IDC_SPN_SEC)->EnableWindow(FALSE);
        GetDlgItem(IDC_SPN_MS)->EnableWindow(FALSE);
        GetDlgItem(IDC_SPN_US)->EnableWindow(FALSE);

		GetDlgItem(IDC_EDT_GAIN)->EnableWindow(FALSE);
		GetDlgItem(IDC_SLD_GAIN)->EnableWindow(FALSE);
		GetDlgItem(IDC_STA_GAIN)->EnableWindow(FALSE);

        GetDlgItem(IDC_BTN_EXPTM)->EnableWindow(FALSE);

        // ȡǰĿֵ
        if (TUCAMRET_SUCCESS == TUCAM_Prop_GetValue(m_opCam.hIdxTUCam, TUIDP_BRIGHTNESS, &dbVal))
        {    
            int nVal = (int)dbVal;
            CString szText;

            szText.Format(_T("%d%%"), nVal);
            GetDlgItem(IDC_EDT_ATTGR)->SetWindowText(szText);    

            m_sldTgr.SetPos(nVal);
        }

		SetTimer(IDT_GET_AE, IDT_SPC_AE, NULL);
    }
    else
    {
        m_sldTgr.EnableWindow(FALSE);
        TUCAM_Capa_SetValue(m_opCam.hIdxTUCam, TUIDC_ATEXPOSURE, 0);

        GetDlgItem(IDC_EDT_ATTGR)->EnableWindow(FALSE);

        GetDlgItem(IDC_STA_SEC)->EnableWindow(TRUE);
        GetDlgItem(IDC_STA_MS)->EnableWindow(TRUE);
        GetDlgItem(IDC_STA_US)->EnableWindow(TRUE);

        GetDlgItem(IDC_EDT_SEC)->EnableWindow(TRUE);
        GetDlgItem(IDC_EDT_MS)->EnableWindow(TRUE);
        GetDlgItem(IDC_EDT_US)->EnableWindow(TRUE);
        GetDlgItem(IDC_SPN_SEC)->EnableWindow(TRUE);
        GetDlgItem(IDC_SPN_MS)->EnableWindow(TRUE);
        GetDlgItem(IDC_SPN_US)->EnableWindow(TRUE);

		GetDlgItem(IDC_EDT_GAIN)->EnableWindow(TRUE);
		GetDlgItem(IDC_SLD_GAIN)->EnableWindow(TRUE);
		GetDlgItem(IDC_STA_GAIN)->EnableWindow(TRUE);

        GetDlgItem(IDC_BTN_EXPTM)->EnableWindow(TRUE);

        // ȡǰعʱ
        if (TUCAMRET_SUCCESS == TUCAM_Prop_GetValue(m_opCam.hIdxTUCam, TUIDP_EXPOSURETM, &dbVal))
        {
            UpdateExposureTime((DWORD)(dbVal * 1000));
        }

		KillTimer(IDT_GET_AE);
    }
}


void CameraControl::OnNMCustomdrawSldRatio(NMHDR *pNMHDR, LRESULT *pResult)
{
	LPNMCUSTOMDRAW pNMCD = reinterpret_cast<LPNMCUSTOMDRAW>(pNMHDR);
	// TODO:  ڴӿؼ֪ͨ
	if (!m_bRatioRange)
	{
		m_bRatioRange = TRUE;
		return;
	}

	if (NULL != m_opCam.hIdxTUCam)
	{
		int nPos = m_sldRatio.GetPos();
		TUCAM_Prop_SetValue(m_opCam.hIdxTUCam, TUIDP_PIXELRATIO, nPos);

		CString szText = _T("");
		szText.Format(_T("%0.2f%%"), nPos / 100.0);
		GetDlgItem(IDC_EDT_RATIO)->SetWindowText(szText);
	}

	*pResult = 0;
}

void CameraControl::OnNMCustomdrawSldTgr(NMHDR *pNMHDR, LRESULT *pResult)
{
	LPNMCUSTOMDRAW pNMCD = reinterpret_cast<LPNMCUSTOMDRAW>(pNMHDR);
	// TODO: Add your control notification handler code here
	 // ԶعĿֵֵֹı
    if (!m_bTargetRange)
    {
        m_bTargetRange = TRUE;
        return;
    }

    if (NULL != m_opCam.hIdxTUCam)
    {
        int nPos = m_sldTgr.GetPos();
        TUCAM_Prop_SetValue(m_opCam.hIdxTUCam, TUIDP_BRIGHTNESS, nPos);          

        CString szText = _T("");
        szText.Format(_T("%d%%"), nPos);
        GetDlgItem(IDC_EDT_ATTGR)->SetWindowText(szText);  
    }
	*pResult = 0;
}

//عʱ䣨ӣ
void CameraControl::OnDeltaposSpnSec(NMHDR *pNMHDR, LRESULT *pResult)
{
	LPNMUPDOWN pNMUpDown = reinterpret_cast<LPNMUPDOWN>(pNMHDR);
	// TODO: ڴӿؼ֪ͨ

	m_uSec = GetDlgItemInt(IDC_EDT_SEC);

	if (pNMUpDown->iDelta < 0)
	{
		if (m_uSec >= 3600)
		{
			return;
		}
		m_uSec++;

		CString strSec;
		strSec.Format(_T("%d"), m_uSec);
		GetDlgItem(IDC_EDT_SEC)->SetWindowText(strSec);
	}
	else if (pNMUpDown->iDelta > 0)
	{
		if (m_uSec <= 0)
		{
			return;
		}
		m_uSec--;

		CString strSec;
		strSec.Format(_T("%d"), m_uSec);
		GetDlgItem(IDC_EDT_SEC)->SetWindowText(strSec);
	}

    OnBnClickedBtnExptm();

	*pResult = 0;
}

//ع¼룩
void CameraControl::OnDeltaposSpnMs(NMHDR *pNMHDR, LRESULT *pResult)
{
	LPNMUPDOWN pNMUpDown = reinterpret_cast<LPNMUPDOWN>(pNMHDR);
	// TODO: ڴӿؼ֪ͨ

	m_uMs = GetDlgItemInt(IDC_EDT_MS);

	if (pNMUpDown->iDelta < 0)
	{
		m_uMs++;

		CString strMs;
		strMs.Format(_T("%d"), m_uMs);
		GetDlgItem(IDC_EDT_MS)->SetWindowText(strMs);
	}
	else if (pNMUpDown->iDelta > 0)
	{

		if (m_uMs >= 0)
		{
			m_uMs--;
		}

		if (m_uSec <= 0 && m_uMs < 0)
		{
			m_uMs = 0;
		}

		CString strMs;
		strMs.Format(_T("%d"), m_uMs);
		GetDlgItem(IDC_EDT_MS)->SetWindowText(strMs);
	}

    OnBnClickedBtnExptm();

	*pResult = 0;
}

//عʱ䣨΢룩
void CameraControl::OnDeltaposSpnUs(NMHDR *pNMHDR, LRESULT *pResult)
{
	LPNMUPDOWN pNMUpDown = reinterpret_cast<LPNMUPDOWN>(pNMHDR);
	// TODO: ڴӿؼ֪ͨ
	m_uUs = GetDlgItemInt(IDC_EDT_US);
	int nUs = 0;
    if (pNMUpDown->iDelta < 0)
	{
		m_uUs += m_uiExpStep;

		CString strUs;
		strUs.Format(_T("%d"), m_uUs);
		GetDlgItem(IDC_EDT_US)->SetWindowText(strUs);
	}
	else if (pNMUpDown->iDelta > 0)
	{
		if (m_uUs >= 0)
		{
			 m_uUs -= m_uiExpStep;
		}
	
		if (m_uSec <= 0 && m_uMs <= 0 && m_uUs < 0)
		{
			m_uUs = 0;
		}

		CString strUs;
		strUs.Format(_T("%d"), m_uUs);
		GetDlgItem(IDC_EDT_US)->SetWindowText(strUs);
	}

    OnBnClickedBtnExptm();

	*pResult = 0;
}

void CameraControl::OnBnClickedBtnExptm()
{
	// TODO: Add your control notification handler code here
	UpdateData();

	if (m_uSec > 3600)
		m_uSec = 3600;	

    DWORD dwExp = m_uSec *1000 * 1000 + m_uMs * 1000 + m_uUs;  

	if (m_nPID == 0xEC07 || m_nPID == 0xEC09)
	{
		if (dwExp < 137)
			dwExp = 130;		

		if (dwExp > 15000000)
			dwExp = 15000000;	
	}
	else if (m_nPID == 0xE407)
	{
		if (dwExp > 14000000)
			dwExp = 14000000;	
	}
	else if (m_nPID == 0xE40A)
	{
		if (dwExp > 12000000)
			dwExp = 12000000;	
	}
	else if (m_nPID == 0xE419)
	{
		if (dwExp > 10000000)
			dwExp = 10000000;
	}
	else if (m_nPID == 0xE420)
	{
		if (dwExp > 300000000)
			dwExp = 300000000;
	}
	else if (m_nPID == FL20BW_PID_SSC183IM1)
	{
		if (dwExp > 3600000000)
			dwExp = 3600000000;	
	}

    double dblExp = (double)dwExp / 1000.0f;

	TUCAM_PROP_ATTR  attrProp;
    if (NULL != m_opCam.hIdxTUCam)
    {

		attrProp.nIdxChn= 0;                    // ʹĬͨ
		attrProp.idProp = TUIDP_EXPOSURETM;
		TUCAM_Prop_GetAttr(m_opCam.hIdxTUCam, &attrProp);
		m_uiExpStep = (UINT)(attrProp.dbValStep * 1000 + 0.5f);

        TUCAM_Prop_SetValue(m_opCam.hIdxTUCam, TUIDP_EXPOSURETM, dblExp);
        TUCAM_Prop_GetValue(m_opCam.hIdxTUCam, TUIDP_EXPOSURETM, &dblExp);

        UpdateExposureTime((DWORD)(dblExp * 1000));
    } 
}

void CameraControl::UpdateExposureTime(DWORD dwExp)
{
	if (m_nPID == 0xEC07 || m_nPID == 0xEC09)
	{
		if (dwExp < 137)
			dwExp = 130;		

		if (dwExp > 15000000)
			dwExp = 15000000;	
	}
	else if (m_nPID == 0xE407)
	{
		if (dwExp > 14000000)
			dwExp = 14000000;	
	}
	else if (m_nPID == 0xE40A)
	{
		if (dwExp > 12000000)
			dwExp = 12000000;	
	}

    m_uSec = (dwExp / (1000 * 1000)) /*% 60*/;
    m_uMs  = (dwExp / 1000) % 1000;
    m_uUs  = dwExp % 1000;

    CString sz;
    sz.Format(_T("%d"), m_uSec);
    GetDlgItem(IDC_EDT_SEC)->SetWindowText(sz);
    sz.Format(_T("%d"), m_uMs);
    GetDlgItem(IDC_EDT_MS)->SetWindowText(sz);
    sz.Format(_T("%d"), m_uUs);
    GetDlgItem(IDC_EDT_US)->SetWindowText(sz);
}

void CameraControl::RefreshRate()
{
	double dbValue = 0;
	CString szText;
	if (NULL != m_opCam.hIdxTUCam)
	{
		if(TUCAMRET_SUCCESS == TUCAM_Prop_GetValue(m_opCam.hIdxTUCam, TUIDP_FRAME_RATE, &dbValue))
		{
			m_dbFrameRate = dbValue;
			szText.Format(_T("%.2f"), m_dbFrameRate);
			GetDlgItem(IDC_EDT_RATE)->SetWindowText(szText); 
		}
	}
}

void CameraControl::OnBnClickedChkBitofdepth()
{
	// TODO: Add your control notification handler code here
    BOOL bWaitting = m_ccmode->m_bWaitting;

    if (bWaitting)
    {
        m_ccmode->StopWaitForFrame();
    }

    if (m_chkBDep.GetCheck())
    {
        // 16bit
        if (NULL != m_opCam.hIdxTUCam)
        {
            TUCAM_Capa_SetValue(m_opCam.hIdxTUCam, TUIDC_BITOFDEPTH, 16);         
        }
    }
    else
    {
        // 8bit
        if (NULL != m_opCam.hIdxTUCam)
        {
            TUCAM_Capa_SetValue(m_opCam.hIdxTUCam, TUIDC_BITOFDEPTH, 8);         
        }
    }

	m_ccmode->OnRefreshLevelRange();
    
	OnBnClickedBtnExptm();

    if (bWaitting)
    {
        m_ccmode->StartWaitForFrame();
		m_ccmode->UpdataPic();
    }
}

void CameraControl::OnBnClickedRadioHdr()
{
	// TODO: Add your control notification handler code here
	m_nGain = 0;

    if (NULL != m_opCam.hIdxTUCam)
    {
        ((CButton *)GetDlgItem(IDC_RADIO_HDR))->SetCheck(TRUE);
        ((CButton *)GetDlgItem(IDC_RADIO_HIGHGAIN))->SetCheck(FALSE);
        ((CButton *)GetDlgItem(IDC_RADIO_LOWGAIN))->SetCheck(FALSE);
        ((CButton *)GetDlgItem(IDC_RADIO_CMS))->SetCheck(FALSE);

		BOOL bWaitting = m_ccmode->m_bWaitting;

		if (bWaitting)
		{
			m_ccmode->StopWaitForFrame();
		}

		SetGain();
		UpdateBitOfDepthEnum();
		m_ccmode->OnRefreshLevelRange();

		if (bWaitting)
		{
			m_ccmode->StartWaitForFrame();
			m_ccmode->UpdataInvalidate();
		}

		if(m_bSprtCMS)
		{
			int nVal = 0;
			if (TUCAMRET_SUCCESS == TUCAM_Capa_GetValue(m_opCam.hIdxTUCam, TUIDC_IMGMODESELECT, &nVal))
			{
				if (0 != nVal)
				{
					TUCAM_Capa_SetValue(m_opCam.hIdxTUCam, TUIDC_IMGMODESELECT, 0);
				}
			}
		}
    }
}

void CameraControl::OnBnClickedRadioHighgain()
{
	// TODO: Add your control notification handler code here
	m_nGain = 1;

    if (NULL != m_opCam.hIdxTUCam)
    {
        ((CButton *)GetDlgItem(IDC_RADIO_HDR))->SetCheck(FALSE);
        ((CButton *)GetDlgItem(IDC_RADIO_HIGHGAIN))->SetCheck(TRUE);
        ((CButton *)GetDlgItem(IDC_RADIO_LOWGAIN))->SetCheck(FALSE);
        ((CButton *)GetDlgItem(IDC_RADIO_CMS))->SetCheck(FALSE);

		BOOL bWaitting = m_ccmode->m_bWaitting;

		if (bWaitting)
		{
			m_ccmode->StopWaitForFrame();
		}

		SetGain();
		UpdateBitOfDepthEnum();
		m_ccmode->OnRefreshLevelRange();

		if (bWaitting)
		{
			m_ccmode->StartWaitForFrame();
			m_ccmode->UpdataInvalidate();
		}

		if(m_bSprtCMS)
		{
			int nVal = 0;
			if (TUCAMRET_SUCCESS == TUCAM_Capa_GetValue(m_opCam.hIdxTUCam, TUIDC_IMGMODESELECT, &nVal))
			{
				if (0 != nVal)
				{
					TUCAM_Capa_SetValue(m_opCam.hIdxTUCam, TUIDC_IMGMODESELECT, 0);
				}
			}
		}
    }
}

void CameraControl::OnBnClickedRadioLowgain()
{
	// TODO: Add your control notification handler code here
	m_nGain = 2;

    if (NULL != m_opCam.hIdxTUCam)
    {
        ((CButton *)GetDlgItem(IDC_RADIO_HDR))->SetCheck(FALSE);
        ((CButton *)GetDlgItem(IDC_RADIO_HIGHGAIN))->SetCheck(FALSE);
        ((CButton *)GetDlgItem(IDC_RADIO_LOWGAIN))->SetCheck(TRUE);
        ((CButton *)GetDlgItem(IDC_RADIO_CMS))->SetCheck(FALSE);

		BOOL bWaitting = m_ccmode->m_bWaitting;

		if (bWaitting)
		{
			m_ccmode->StopWaitForFrame();
		}

        SetGain();
		UpdateBitOfDepthEnum();
		m_ccmode->OnRefreshLevelRange();

		if (bWaitting)
		{
			m_ccmode->StartWaitForFrame();
			m_ccmode->UpdataInvalidate();
		}

		if(m_bSprtCMS)
		{
			int nVal = 0;
			if (TUCAMRET_SUCCESS == TUCAM_Capa_GetValue(m_opCam.hIdxTUCam, TUIDC_IMGMODESELECT, &nVal))
			{
				if (0 != nVal)
				{
					TUCAM_Capa_SetValue(m_opCam.hIdxTUCam, TUIDC_IMGMODESELECT, 0);
				}
			}
		}
    }

}

void CameraControl::OnNMCustomdrawSldGain(NMHDR *pNMHDR, LRESULT *pResult)
{
    LPNMCUSTOMDRAW pNMCD = reinterpret_cast<LPNMCUSTOMDRAW>(pNMHDR);
    // TODO: ڴӿؼ֪ͨ

    // 棬ֵֹı
    if (!m_bGainRange)
    {
        m_bGainRange = TRUE;
        return;
    }

    if (NULL != m_opCam.hIdxTUCam)
    {
        int nPos = m_sldGain.GetPos();
        TUCAM_Prop_SetValue(m_opCam.hIdxTUCam, TUIDP_GLOBALGAIN, nPos);          

        CString szText = _T("");
        szText.Format(_T("%d"), nPos);
        GetDlgItem(IDC_EDT_GAIN)->SetWindowText(szText);  
    }


    *pResult = 0;
}

int CameraControl::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
    if (CDialog::OnCreate(lpCreateStruct) == -1)
        return -1;

    // TODO:  ڴרõĴ

    SetTimer(IDT_GET_TEMPERATURE, IDT_SPC_TEMPERATURE, NULL);

    return 0;
}

void CameraControl::OnDestroy()
{
    CDialog::OnDestroy();

    // TODO: ڴ˴Ϣ

    KillTimer(IDT_GET_TEMPERATURE);
}

void CameraControl::OnBnClickedBtnOnceae()
{
    // TODO: ڴӿؼ֪ͨ

    GetDlgItem(IDC_BTN_ONCEAE)->EnableWindow(FALSE);

    if (NULL != m_opCam.hIdxTUCam)
    {
        if (TUCAMRET_SUCCESS == TUCAM_Capa_SetValue(m_opCam.hIdxTUCam, TUIDC_ATEXPOSURE, 2))
        {
            SetTimer(IDT_GET_ONCE_AE, IDT_SPC_ONCE_AE, NULL);

            GetDlgItem(IDC_EDT_ATTGR)->EnableWindow(TRUE);

            GetDlgItem(IDC_STA_SEC)->EnableWindow(FALSE);
            GetDlgItem(IDC_STA_MS)->EnableWindow(FALSE);
            GetDlgItem(IDC_STA_US)->EnableWindow(FALSE);

            GetDlgItem(IDC_EDT_SEC)->EnableWindow(FALSE);
            GetDlgItem(IDC_EDT_MS)->EnableWindow(FALSE);
            GetDlgItem(IDC_EDT_US)->EnableWindow(FALSE);
            GetDlgItem(IDC_SPN_SEC)->EnableWindow(FALSE);
            GetDlgItem(IDC_SPN_MS)->EnableWindow(FALSE);
            GetDlgItem(IDC_SPN_US)->EnableWindow(FALSE);

            GetDlgItem(IDC_BTN_EXPTM)->EnableWindow(FALSE);

            double dbVal = 0.0f;

            // ȡǰĿֵ
            if (TUCAMRET_SUCCESS == TUCAM_Prop_GetValue(m_opCam.hIdxTUCam, TUIDP_BRIGHTNESS, &dbVal))
            {    
                int nVal = (int)dbVal;
                CString szText;

                szText.Format(_T("%d%%"), nVal);
                GetDlgItem(IDC_EDT_ATTGR)->SetWindowText(szText);    

                m_sldTgr.SetPos(nVal);
            }
        }
        else
        {
            GetDlgItem(IDC_BTN_ONCEAE)->EnableWindow(TRUE);
        }
    }
}

void CameraControl::OnBnClickedCms()
{
    // TODO: ڴӿؼ֪ͨ

    m_nGain = 0;

    if (NULL != m_opCam.hIdxTUCam)
    {
        ((CButton *)GetDlgItem(IDC_RADIO_HDR))->SetCheck(FALSE);
        ((CButton *)GetDlgItem(IDC_RADIO_HIGHGAIN))->SetCheck(FALSE);
        ((CButton *)GetDlgItem(IDC_RADIO_LOWGAIN))->SetCheck(FALSE);
        ((CButton *)GetDlgItem(IDC_RADIO_CMS))->SetCheck(TRUE);
        
        TUCAM_Prop_SetValue(m_opCam.hIdxTUCam, TUIDP_GLOBALGAIN, 0);

        int nVal = 0;
        if (TUCAMRET_SUCCESS == TUCAM_Capa_GetValue(m_opCam.hIdxTUCam, TUIDC_IMGMODESELECT, &nVal))
        {
            if (1 != nVal)
            {
                TUCAM_Capa_SetValue(m_opCam.hIdxTUCam, TUIDC_IMGMODESELECT, 1);
            }
        }
    }
}

void CameraControl::OnBnClickedBtnUsbled()
{
	// TODO: ڴӿؼ֪ͨ
	if (NULL != m_opCam.hIdxTUCam)
	{
		m_bLedState  = !m_bLedState;

		TUCAM_Capa_SetValue(m_opCam.hIdxTUCam, TUIDC_ENABLELED, m_bLedState);

		LedShow(m_bLedState);
	}
}

void CameraControl::OnBnClickedBtnPienable()
{
	// TODO: ڴӿؼ֪ͨ
	if (NULL != m_opCam.hIdxTUCam)
	{
		m_bPIState  = !m_bPIState;

		TUCAM_Capa_SetValue(m_opCam.hIdxTUCam, TUIDC_ENABLEPI, m_bPIState);

		PIEnableShow(m_bPIState);
	}
}

void CameraControl::OnCbnSelchangeCmbBitofdepth()
{
	// TODO: ڴӿؼ֪ͨ
	UpdateData();

	int nVal = m_cmbBitOfDepth.GetCurSel();

	if (NULL != m_opCam.hIdxTUCam)
	{
		BOOL bWaitting = m_ccmode->m_bWaitting;

		if (bWaitting)
		{
			m_ccmode->StopWaitForFrame();
		}

		TUCAM_Capa_SetValue(m_opCam.hIdxTUCam, TUIDC_BITOFDEPTH, nVal);
		m_ccmode->OnRefreshLevelRange();
		RefreshRate();
		if (bWaitting)
		{
			m_ccmode->StartWaitForFrame();
			m_ccmode->UpdataInvalidate();
		}
	}
	
}

void CameraControl::OnBnClickedBtnRate()
{
	// TODO: ڴӿؼ֪ͨ
	UpdateData();
	double dbValue = m_dbFrameRate;
	TUCAM_Prop_SetValue(m_opCam.hIdxTUCam, TUIDP_FRAME_RATE, dbValue); 
	RefreshRate();
}

void CameraControl::OnCbnSelchangeCmbTestimg()
{
	// TODO: ڴӿؼ֪ͨ
	TUCAM_Capa_SetValue(m_opCam.hIdxTUCam, TUIDC_TESTIMGMODE, m_cmbTestImg.GetCurSel());
}

void CameraControl::OnBnClickedBtnDsnu()
{
	// TODO: ڴӿؼ֪ͨ
	if (NULL != m_opCam.hIdxTUCam)
	{
		m_bDSNUState  = !m_bDSNUState;

		if (TUCAMRET_SUCCESS != TUCAM_Capa_SetValue(m_opCam.hIdxTUCam, TUIDC_ENABLEDSNU, m_bDSNUState))
		{
			SetGain();
		}

		DSNUEnableShow(m_bDSNUState);
	}
}

void CameraControl::OnBnClickedBtnPrnu()
{
	// TODO: ڴӿؼ֪ͨ
	if (NULL != m_opCam.hIdxTUCam)
	{
		m_bPRNUState  = !m_bPRNUState;

		TUCAM_Capa_SetValue(m_opCam.hIdxTUCam, TUIDC_ENABLEIMGPRO, m_bPRNUState ? 0x0E : 0x0A);

		PRNUEnableShow(m_bPRNUState);
	}
}

void CameraControl::OnCbnSelchangeCmbFan()
{
	// TODO:  ڴӿؼ֪ͨ
	if (NULL != m_opCam.hIdxTUCam)
	{
		TUCAM_Capa_SetValue(m_opCam.hIdxTUCam, TUIDC_FAN_GEAR, m_cmbFan.GetCurSel());
	}
}

void CameraControl::TECShow(BOOL bVal)
{
	if (bVal)
	{
		GetDlgItem(IDC_BTN_TEC)->SetWindowText(_T("TEC On") );
	}
	else
	{
		GetDlgItem(IDC_BTN_TEC)->SetWindowText(_T("TEC Off"));
	}

	GetDlgItem(IDC_STA_COOLTARGET)->EnableWindow(bVal);
	GetDlgItem(IDC_SLD_COOLTARGET)->EnableWindow(bVal);
	GetDlgItem(IDC_EDT_COOLTARGET)->EnableWindow(bVal);
}

void CameraControl::OnBnClickedBtnTec()
{
	// TODO:  ڴӿؼ֪ͨ
	m_bTEC = !m_bTEC;

	if (m_bTEC)
	{
		int nVal = ::MessageBox(0, _T("Please make sure that the device is connected to a vacuum environment"), _T("SamplePro"), MB_YESNO);

		if (nVal == IDNO)
		{
			m_bTEC = FALSE;
			return;
		}
	}

	if (NULL != m_opCam.hIdxTUCam)
	{
		TUCAM_Capa_SetValue(m_opCam.hIdxTUCam, TUIDC_ENABLETEC, (INT)m_bTEC);
	}

	TECShow(m_bTEC);
}

void CameraControl::OnNMCustomdrawSldCooltarget(NMHDR *pNMHDR, LRESULT *pResult)
{
	LPNMCUSTOMDRAW pNMCD = reinterpret_cast<LPNMCUSTOMDRAW>(pNMHDR);
	// TODO:  ڴӿؼ֪ͨ
	if (!m_bTempChange)
	{
		m_bTempChange = TRUE;
		return;
	}

	if (NULL != m_opCam.hIdxTUCam)
	{
		CString szTemp;
		int nVal = m_sldCoolT.GetPos();

		if (m_nMidTTemp > 100)
		{
			szTemp.Format(_T("%.1f "), (nVal - m_nMidTTemp) / 10.0);
		}
		else
		{
			szTemp.Format(_T("%d "), (nVal - m_nMidTTemp));
		}
		
		GetDlgItem(IDC_EDT_COOLTARGET)->SetWindowText(szTemp);

//      m_fValTemp = (float)((nVal - m_nMidTTemp));
		TUCAM_Prop_SetValue(m_opCam.hIdxTUCam, TUIDP_TEMPERATURE, nVal);
	}
	*pResult = 0;
}

void CameraControl::OnCbnSelchangeCmbGain()
{
	// TODO:  ڴӿؼ֪ͨ
	if (NULL != m_opCam.hIdxTUCam)
	{
		BOOL bWaitting = m_ccmode->m_bWaitting;

		if (bWaitting)
		{
			m_ccmode->StopWaitForFrame();
		}

		int value = m_cmbGain.GetCurSel();
		TUCAM_Prop_SetValue(m_opCam.hIdxTUCam, TUIDP_GLOBALGAIN, value);

		double dbVal = 0;
		// ȡǰعʱ
		if (TUCAMRET_SUCCESS == TUCAM_Prop_GetValue(m_opCam.hIdxTUCam, TUIDP_EXPOSURETM, &dbVal))
		{
			UpdateExposureTime((DWORD)(dbVal * 1000));
		}

		m_ccmode->OnRefreshLevelRange();
		RefreshRate();
		if (bWaitting)
		{
			m_ccmode->StartWaitForFrame();
			m_ccmode->UpdataPic();
		}
	}
}

void CameraControl::OnCbnSelchangeCmbShutter()
{
	// TODO:  ڴӿؼ֪ͨ
	if (NULL != m_opCam.hIdxTUCam)
	{
		int value = m_cmbShutter.GetCurSel();
		TUCAM_Capa_SetValue(m_opCam.hIdxTUCam, TUIDC_SHUTTER, value);
		RefreshRate();

		if (m_ccmode->m_bWaitting)
		{
			if (TUCCM_SEQUENCE != m_ccmode->m_nTriMode)
			{
				((Trigger*)m_ccmode->m_tabPage[4])->EnableTriggerControl(TRUE);
			}
		}
	}
}

void CameraControl::OnBnClickedChkTimestamp()
{
	// TODO:  ڴӿؼ֪ͨ
	if (NULL != m_opCam.hIdxTUCam)
	{
		BOOL bWaitting = m_ccmode->m_bWaitting;

		if (bWaitting)
		{
			m_ccmode->StopWaitForFrame();
		}
		m_ccmode->m_bTimeStamp = m_chkTimestamp.GetCheck();
		TUCAM_Capa_SetValue(m_opCam.hIdxTUCam, TUIDC_ENABLETIMESTAMP, m_ccmode->m_bTimeStamp);

		if (TUCCM_SEQUENCE == m_ccmode->m_nTriMode)
		{
			GetDlgItem(IDC_BTN_ONCEAE)->EnableWindow(m_ccmode->m_bTimeStamp);
		}

		if (bWaitting)
		{
			m_ccmode->StartWaitForFrame();
			m_ccmode->UpdataPic();
		}
	}
}

void CameraControl::OnCbnSelchangeCmbImgselect()
{
	// TODO:  ڴӿؼ֪ͨ
	UpdateData();

	int nVal = m_ccmode->m_nImgMode;
	int nCur = (m_cmbImgSelect.GetCurSel() + m_nImgModeMin);

	if (NULL != m_opCam.hIdxTUCam)
	{
		BOOL bWaitting = m_ccmode->m_bWaitting;

		if (bWaitting)
		{
			m_ccmode->StopWaitForFrame();
		}

		TUCAM_PROP_ATTR  attrProp;
		if (nCur != nVal)
		{
			m_ccmode->m_nImgMode = nCur;

			DWORD dwExp = m_uSec * 1000 * 1000 + m_uMs * 1000 + m_uUs;
			double dblExp = dwExp / 1000.0f;

			GainModeEnable();
			GainModeShow();
			SetGain();
			m_ccmode->OnRefreshLevelRange();

			TUCAM_Prop_SetValue(m_opCam.hIdxTUCam, TUIDP_EXPOSURETM, dblExp);
			TUCAM_Prop_GetValue(m_opCam.hIdxTUCam, TUIDP_EXPOSURETM, &dblExp);

			UpdateExposureTime((DWORD)(dblExp * 1000));
			RefreshRate();
			attrProp.nIdxChn = 0;                    // ʹĬͨ
			attrProp.idProp = TUIDP_EXPOSURETM;
			TUCAM_Prop_GetAttr(m_opCam.hIdxTUCam, &attrProp);
			m_uiExpStep = (UINT)(attrProp.dbValStep * 1000 + 0.5f);
			((Trigger*)m_ccmode->m_tabPage[4])->RefreshGlobalreset();
		}

		if (bWaitting)
		{
			m_ccmode->StartWaitForFrame();
			m_ccmode->UpdataInvalidate();
		}
	}
}

void CameraControl::UpdateBitOfDepthEnum()
{
	if (!m_ccmode->IsDhyana401DNew())
	{
		return;
	}

	int i = 0;
	int nVal = 0;
	CString szTxt;
	char szDepth[64] = { 0 };

	TUCAM_CAPA_ATTR  attrCapa;
	TUCAM_VALUE_TEXT valText;
	
	valText.nTextSize = 64;
	valText.pText = &szDepth[0];

	attrCapa.idCapa = TUIDC_BITOFDEPTH;
	if (TUCAMRET_SUCCESS == TUCAM_Capa_GetAttr(m_opCam.hIdxTUCam, &attrCapa))
	{
		TUCAM_Capa_GetValue(m_opCam.hIdxTUCam, TUIDC_BITOFDEPTH, &nVal);
		int nCnt = attrCapa.nValMax - attrCapa.nValMin + 1;

		valText.nID = TUIDC_BITOFDEPTH;
		m_cmbBitOfDepth.ResetContent();

		for (i = 0; i < nCnt; ++i)
		{
			valText.dbValue = i;
			TUCAM_Capa_GetValueText(m_opCam.hIdxTUCam, &valText);
			szTxt = valText.pText;
			m_cmbBitOfDepth.InsertString(i, szTxt);
		}

		m_cmbBitOfDepth.SetCurSel(nVal);
	}
}