#include "stdafx.h"
#include "ImageSaveManager.h"
#include "MemoryManager.h"

#define MAKETAG(a,b,c,d) (a | (b << 8) | (c << 16) | (d << 24))
#define  TUDBG_PRINTF(format,...)	{char dbgMsg[1024] = {0}; sprintf_s(dbgMsg, "" format"", ##__VA_ARGS__); OutputDebugStringA(dbgMsg);}

ImageSaveManager *ImageSaveManager::instance = NULL;

ImageSaveManager *ImageSaveManager::GetInstance()
{
	if (instance == NULL)
	{
		instance = new ImageSaveManager();
	}
	return instance;
}

ImageSaveManager::ImageSaveManager()
{
	storageMediumType = DISK;
	captureFlag = false;
	triggerFlag = false;
	recordFlag = false;
	saveFormat = TUFMT_TIF;

	videoCompressorHandle = MAKETAG('D', 'I', 'B', ' ');
	videoCompressorHandleIndex = 0xFF;
	recordTimes = 0;
	fps = 10;
	imageRAMToDiskSaveThreadHandle = NULL;

	currentSaveType = IMAGE;

	saveTimes = 0;
	saveFrameIndex = 1;
	imageNamePrefix = "TS";
	totalFrame = 1;
	InitSavePath();
}

ImageSaveManager::~ImageSaveManager()
{
}

void ImageSaveManager::InitSavePath()
{
	CString path;
	GetCurrentDirectory(MAX_PATH, path.GetBuffer(MAX_PATH));
	strcat(path.GetBuffer(MAX_PATH), "\\img");
	strcpy(savePath, path.GetBuffer(MAX_PATH));
}

void ImageSaveManager::CreatePath(char *path)
{	
	if (!PathIsDirectory(path))
	{
		CreateDirectory(path, NULL);
	}
}

void ImageSaveManager::SetRAWChecked(bool checked)
{
	if (checked)
	{
		saveFormat |= TUFMT_RAW;
	}
	else
	{
		saveFormat &= ~TUFMT_RAW;
	}
}

void ImageSaveManager::SetTIFChecked(bool checked)
{
	if (checked)
	{
		saveFormat |= TUFMT_TIF;
	}
	else
	{
		saveFormat &= ~TUFMT_TIF;
	}
}

void ImageSaveManager::StartCapture()
{
	if (totalFrame < 1 || GetSaveFormatCount() < 1)
	{
		return;
	}

	currentSaveType = IMAGE;
	if (!captureFlag)
	{
		captureFlag = true;
		++saveTimes;
		saveFrameIndex = 1;
	}
}

int ImageSaveManager::GetSaveFormatCount()
{
	int count = 0;
	if (saveFormat & TUFMT_RAW)
	{
		count++;
	}

	if (saveFormat & TUFMT_TIF)
	{
		count++;
	}
	return count;
}

void ImageSaveManager::Save(TUCAM_FRAME &srcFrame)
{
	if (!captureFlag && !recordFlag && !triggerFlag)
	{
		return;
	}

	if (storageMediumType == DISK || triggerFlag)
	{
		SaveToDisk(srcFrame);
	}
	else
	{
		SaveToRAM(srcFrame);
	}
}

void ImageSaveManager::SaveToDisk(TUCAM_FRAME &srcFrame)
{
	if (recordFlag)
	{
		TUCAM_Rec_AppendFrame(cameraHandle, &srcFrame);
	}

	if (captureFlag || triggerFlag)
	{
		bool succeedFlag = SaveImage(srcFrame);
		if (!succeedFlag)
		{
			return;
		}

		saveFrameIndex++;
		if (saveFrameIndex > totalFrame && !triggerFlag)// ս
		{
			captureFlag = FALSE;
			//pIn->m_tabPage[2]->PostMessage(TUMSG_CAP, NULL, (LPARAM)pIn->m_nCntSave);
		}
	}
}

bool ImageSaveManager::SaveImage(TUCAM_FRAME &srcFrame)
{
	CreatePath(savePath);

	CString szFName;
	szFName.Format(_T("%s\\%s_%d_%d"), savePath, imageNamePrefix, saveTimes, saveFrameIndex);
	int nFmt = saveFormat;
	TUCAM_FRAME frame;
	TUCAM_FILE_SAVE fileSave;

	fileSave.nSaveFmt = nFmt;
	memcpy(&frame, &srcFrame, sizeof(TUCAM_FRAME));
	fileSave.pFrame = &frame;
	fileSave.pstrSavePath = (PCHAR)szFName.GetBuffer(0);  // path

	// Format RAW
	if (nFmt & TUFMT_RAW)
	{
		nFmt &= ~TUFMT_RAW;
	}

	if (0 != nFmt)
	{
		fileSave.nSaveFmt = nFmt;

		//if (pIn->IsDhyana400BSIV3())
		{
			srcFrame.ucFormatGet = TUFRM_FMT_RAW;
			if (TUCAMRET_SUCCESS != TUCAM_Buf_CopyFrame(cameraHandle, &srcFrame))
			{
				return false;
			}
		}
		// Save other format data(TIFF/BMP/PNG/JPEG)
		if (TUCAMRET_SUCCESS != TUCAM_File_SaveImage(cameraHandle, fileSave))
		{
			return false;
		}
	}

	if (saveFormat & TUFMT_RAW)
	{
		fileSave.nSaveFmt = TUFMT_RAW;

		// Get RAW data
		srcFrame.ucFormatGet = TUFRM_FMT_RAW;
		if (TUCAMRET_SUCCESS != TUCAM_Buf_CopyFrame(cameraHandle, &srcFrame))
		{
			return false;
		}

		char path[1024];
		sprintf_s(path, "%s\\%s_%d_%d-offset_%d-w_%d-h_%d-Mono%d", savePath, imageNamePrefix, saveTimes, saveFrameIndex, srcFrame.usHeader, srcFrame.usWidth, srcFrame.usHeight, (2 == srcFrame.ucElemBytes ? 16 : 8));
		fileSave.pstrSavePath = path;
		// Save RAW data
		if (TUCAMRET_SUCCESS != TUCAM_File_SaveImage(cameraHandle, fileSave))
		{
			return false;
		}
	}
	return true;
}

void ImageSaveManager::SaveToRAM(TUCAM_FRAME &srcFrame)
{
	if (saveFrameIndex >= m_vecBuf.size())
	{
		return;
	}

	TUDBG_PRINTF("ram saveFrameIndex: %d", saveFrameIndex);
	UCHAR *pSrc = m_vecBuf[saveFrameIndex - 1];
	memcpy(pSrc, srcFrame.pBuffer, imageSize);
	++saveFrameIndex;

	if (saveFrameIndex > totalFrame)
	{
		if (currentSaveType == IMAGE)
		{
			StartImageRAMToDiskSaveThread();
			captureFlag = false;
		}
	}
}

void ImageSaveManager::StartImageRAMToDiskSaveThread()
{
	if (imageRAMToDiskSaveThreadHandle != NULL)
	{
		return;
	}
	if (m_vecBuf.size() <= 0)
	{
		return;
	}

	imageRAMToDiskSaveThreadHandle = CreateEvent(NULL, TRUE, FALSE, NULL);
	_beginthread(ImageRAMToDiskSaveThread, 0, this);
}

void ImageSaveManager::ImageRAMToDiskSaveThread(LPVOID param)
{
	ImageSaveManager *imageSaveManager = (ImageSaveManager *)param;
	if (imageSaveManager == NULL)
	{
		return;
	}

	imageSaveManager->ImageRAMToDiskSave();
	SetEvent(imageSaveManager->imageRAMToDiskSaveThreadHandle);
	CloseHandle(imageSaveManager->imageRAMToDiskSaveThreadHandle);
	imageSaveManager->imageRAMToDiskSaveThreadHandle = NULL;
}

void ImageSaveManager::ImageRAMToDiskSave()
{
	if (currentSaveType == VIDEO)
	{
		CreateVideoFileFromImageList();
	}
	else
	{
		SaveImageListToDisk();
	}
}

void ImageSaveManager::CreateVideoFileFromImageList()
{
	CreateVideoFile();

	UCHAR *pSrc = NULL;
	PTUCAM_FRAME pFrame = (PTUCAM_FRAME)m_vecBuf[0];
	for (int i = 0; i < saveFrameIndex; ++i)
	{
		pSrc = m_vecBuf[i];
		memcpy(pFrame, pSrc, sizeof(TUCAM_FRAME));
		pFrame->pBuffer = pSrc;
		TUCAM_Rec_AppendFrame(cameraHandle, pFrame);
	}

	TUCAM_Rec_Stop(cameraHandle);
}

bool ImageSaveManager::CreateVideoFile()
{
	CreatePath(savePath);

	char videoPath[MAX_PATH];
	sprintf_s(videoPath, "%s\\%s_%d.avi", savePath, imageNamePrefix, recordTimes);

	// If File exists, delete it 
	CFileFind fileFind;

	if (fileFind.FindFile(videoPath))
	{
		CFile::Remove(videoPath);
	}

	TUCAM_REC_SAVE recSave;
	recSave.fFps = fps / 1000;
	recSave.nCodec = ImageSaveManager::GetInstance()->GetVideoCompressorHandle();
	recSave.pstrSavePath = videoPath;

	if (TUCAMRET_SUCCESS == TUCAM_Rec_Start(cameraHandle, recSave))
	{
		return true;
	}
	return false;
}

void ImageSaveManager::SaveImageListToDisk()
{
	UCHAR *pSrc = NULL;
	PTUCAM_FRAME pFrame = (PTUCAM_FRAME)m_vecBuf[0];
	int size = saveFrameIndex - 1;
	saveFrameIndex = 1;
	for (int i = 0; i < size; ++i)
	{
		pSrc = m_vecBuf[i];
		memcpy(pFrame, pSrc, sizeof(TUCAM_FRAME));
		pFrame->pBuffer = pSrc;

		SaveImage(*pFrame);
		++saveFrameIndex;
	}
}

void ImageSaveManager::StartRecord()
{
	currentSaveType = VIDEO;
	if (storageMediumType == DISK)
	{
		RecordToDisk();
	}
	else
	{
		RecordToRAM();
	}
}

void ImageSaveManager::RecordToDisk()
{
	if (recordFlag)
	{
		recordFlag = false;
		TUCAM_Rec_Stop(cameraHandle);
	}
	else
	{
		++recordTimes;
		recordFlag = CreateVideoFile();
	}
}

void ImageSaveManager::RecordToRAM()
{
	if (recordFlag)
	{
		recordFlag = false;
		StartImageRAMToDiskSaveThread();
	}
	else
	{
		++recordTimes;
		recordFlag = true;
	}
}

void ImageSaveManager::SetStorageMediumType(int type)
{
	storageMediumType = type; 
	if (storageMediumType == DISK)
	{
		ReleaseRAMBufList();
	}
	else
	{
		AllocateRAMBufList();
	}
}

void ImageSaveManager::AllocateRAMBufList()
{
	if (imageSize <= 0)
	{
		return;
	}

	ReleaseRAMBufList();
	int num = MemoryManager::GetInstance()->CalculateFrameNumber();
	for (int i = 0; i < num; ++i)
	{
		UCHAR *data = new UCHAR[imageSize];
		m_vecBuf.push_back(data);
	}
}

void ImageSaveManager::ReleaseRAMBufList()
{
	for (int i = m_vecBuf.size() - 1; i >= 0; --i)
	{
		TUCAM_DELBUF(m_vecBuf[i]);
	}
	m_vecBuf.clear();
}