#pragma once

#include "../sdk/inc/TUCamApi.h"
#include <vector>
using namespace std;

class ImageSaveManager
{
public:
	enum SAVE_TYPE
	{
		IMAGE = 0,
		VIDEO = 1,
	};
	enum STORAGE_MEDIUM_TYPE
	{
		DISK = 0,
		RAM = 1,
	};

	~ImageSaveManager();
	static ImageSaveManager *GetInstance();

	void SetCameraHandle(HDTUCAM h) { cameraHandle = h; }

	void SetSavePath(char *path) { strcpy(savePath, path); }
	char *GetSavePath() { return savePath; }

	void SetImageNamePrefix(char *name) { imageNamePrefix = name; }
	char *GetImageNamePrefix() { return imageNamePrefix; }

	bool IsRAWChecked() { return saveFormat & TUFMT_RAW; }
	void SetRAWChecked(bool checked = true);
	bool IsTIFChecked() { return saveFormat & TUFMT_TIF; }
	void SetTIFChecked(bool checked = true);

	void SetTotalFrames(int num) { totalFrame = num; }
	int GetTotalFrames() { return totalFrame; }

	int GetVideoCompressorHandleIndex() { return videoCompressorHandleIndex; }
	void SetVideoCompressorHandleIndex(int index) { videoCompressorHandleIndex = index; }
	DWORD GetVideoCompressorHandle() { return videoCompressorHandle; }
	void SetVideoCompressorHandle(DWORD handler) { videoCompressorHandle = handler; }
	void SetFps(float value) { fps = value; }

	void SetStorageMediumType(int type);
	void AllocateRAMBufList();
	void SetImageSize(int size) { imageSize = size; }

	void StartCapture();
	void StopCapture() { captureFlag = false; }
	bool IsCapturing() { return captureFlag; }
	void Save(TUCAM_FRAME &frame);
	void SetTriggerFlag(bool trigger) { triggerFlag = trigger; }

	void StartRecord();
	bool IsRecording() { return recordFlag; }

	int GetTotalSaveFrameCount() { return totalFrame * GetSaveFormatCount(); }
	int GetHasSavedFrameCount() { return saveFrameIndex * GetSaveFormatCount(); }

private:
	static ImageSaveManager *instance;
	HDTUCAM cameraHandle;

	char savePath[MAX_PATH];
	char *imageNamePrefix;

	int storageMediumType;
	int saveFormat;
	int totalFrame;

	bool currentSaveType;
	bool captureFlag;
	bool triggerFlag;
	int saveTimes;
	int saveFrameIndex;

	bool recordFlag;
	int recordTimes;
	DWORD videoCompressorHandle;
	int videoCompressorHandleIndex;
	float fps;

	vector<UCHAR *> m_vecBuf;
	int imageSize;
	HANDLE imageRAMToDiskSaveThreadHandle;

	ImageSaveManager();
	void InitSavePath();
	void CreatePath(char *path);
	int GetSaveFormatCount();

	void SaveToDisk(TUCAM_FRAME &srcFrame);
	bool SaveImage(TUCAM_FRAME &srcFrame);
	void SaveToRAM(TUCAM_FRAME &srcFrame);

	void RecordToDisk();
	void RecordToRAM();
	void ReleaseRAMBufList();
	
	void StartImageRAMToDiskSaveThread();
	static void ImageRAMToDiskSaveThread(LPVOID param);
	void ImageRAMToDiskSave();
	void CreateVideoFileFromImageList();
	bool CreateVideoFile();
	void SaveImageListToDisk();
};

