#include "stdafx.h"
#include "MemoryManager.h"

#define UNIT1GBYTE 1073741824

MemoryManager *MemoryManager::instance = NULL;

MemoryManager *MemoryManager::GetInstance()
{
	if (instance == NULL)
	{
		instance = new MemoryManager();
	}
	return instance;
}

MemoryManager::MemoryManager()
{
	UpdateCurrentPhysicalMemory();
	memorySetMode = ACTUAL;
	actualPhysicalMemoryGB = 1;
	physicalMemoryPercent = 70;
	imageSize = 10000000;
}

MemoryManager::~MemoryManager()
{
}

void MemoryManager::UpdateCurrentPhysicalMemory()
{
	GetSystemMemorySize(totalPhysicalMemory, availablePhysicalMemory);
}

double MemoryManager::GetTotalPhysicalMemoryGB()
{
	return totalPhysicalMemory * 1.0 / UNIT1GBYTE;
}

double MemoryManager::GetAvailablePhysicalMemoryGB()
{
	return availablePhysicalMemory * 0.7 / UNIT1GBYTE;
}

double MemoryManager::GetActualPhysicalMemoryGB()
{
	if (actualPhysicalMemoryGB < 0)
	{
		actualPhysicalMemoryGB = GetAvailablePhysicalMemoryGB();
	}
	actualPhysicalMemoryGB = min(actualPhysicalMemoryGB, GetAvailablePhysicalMemoryGB());
	return actualPhysicalMemoryGB;
}

BOOL MemoryManager::GetSystemMemorySize(UINT64 &uiMemTotal, UINT64 &uiMemFree)
{
	MEMORYSTATUSEX memsStat;
	memsStat.dwLength = sizeof(memsStat);

	if (!GlobalMemoryStatusEx(&memsStat))
	{
		uiMemTotal = 0;
		uiMemFree = 0;
		return FALSE;
	}

	uiMemFree = memsStat.ullAvailPhys;
	uiMemTotal = memsStat.ullTotalPhys;

#ifndef _WIN64
	uiMemFree = (UINT64)(memsStat.ullAvailVirtual * 0.7f);
	uiMemFree = min(uiMemFree, UNIT1GBYTE);                                                       // 1GB

	//  UINT64 uiTotal = TUMIN(uiMemTotal, 3221225472);                                                 // 3GB-3.75GB
	//  uiMemFree = TUMIN(memsStat.ullAvailVirtual, TUMAX(0, (uiTotal - (uiMemTotal - uiMemFree))));    // 4GB
#endif

	return TRUE;
}

int MemoryManager::CalculateFrameNumber()
{
	double memoryGB = actualPhysicalMemoryGB;
	if (memorySetMode != ACTUAL)
	{
		memoryGB = physicalMemoryPercent / 100.0 * GetAvailablePhysicalMemoryGB();
	}

	UINT64 memory = memoryGB * UNIT1GBYTE;
	return max(memory / imageSize, 1);
}