#include "stdafx.h"
#include "SamplePro.h"
#include "Raminfo.h"
#include "MemoryManager.h"

IMPLEMENT_DYNAMIC(Raminfo, CDialog)

Raminfo::Raminfo(CWnd* pParent)
	: CDialog(Raminfo::IDD, pParent)
	, m_fActual(1.00)
	, m_nPercent(70)
	, frameNumber(0)
{
}

Raminfo::~Raminfo()
{
}

void Raminfo::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Text(pDX, IDC_EDT_ACTUAL, m_fActual);
	DDX_Text(pDX, IDC_EDT_PERCENT, m_nPercent);
	DDV_MinMaxUInt(pDX, m_nPercent, 0, 100);
	DDX_Text(pDX, IDC_STATIC_FRAMES, frameNumber);
}

BEGIN_MESSAGE_MAP(Raminfo, CDialog)
	ON_BN_CLICKED(IDC_RADIO_ACTUAL, &Raminfo::OnBnClickedRadioActual)
	ON_BN_CLICKED(IDC_RADIO_PERCENT, &Raminfo::OnBnClickedRadioPercent)
	ON_EN_CHANGE(IDC_EDT_PERCENT, &Raminfo::OnEnChangeEdtPercent)
	ON_NOTIFY(UDN_DELTAPOS, IDC_SPN_PERCENT, &Raminfo::OnDeltaposSpnPercent)
	ON_BN_CLICKED(IDOK, &Raminfo::OnBnClickedOk)
END_MESSAGE_MAP()

BOOL Raminfo::OnInitDialog()
{
	CDialog::OnInitDialog();

	MemoryManager::GetInstance()->UpdateCurrentPhysicalMemory();
	m_fActual = MemoryManager::GetInstance()->GetActualPhysicalMemoryGB();
	m_nPercent = MemoryManager::GetInstance()->GetPhysicalMemoryPercent();
	UpdateUI();

	CString text;
	float total = MemoryManager::GetInstance()->GetTotalPhysicalMemoryGB();
	float avaiable = MemoryManager::GetInstance()->GetAvailablePhysicalMemoryGB();
	text.Format(_T("Total Physical[Mem=%.2fGB] (%.2fGB Available)"), total, avaiable);
	GetDlgItem(IDC_STATIC_RAMINFO)->SetWindowText(text);

	return true;
}

void Raminfo::UpdateUI()
{
	UpdateData(FALSE);
	EnabledMemorySetControl();
	UpdateFrameNumber();
	SetActualEditText();
}

void Raminfo::EnabledMemorySetControl()
{
	int mode = MemoryManager::GetInstance()->GetMemorySetMode();
	
	bool enable = true;
	if (mode == MemoryManager::ACTUAL)
	{
		((CButton *)GetDlgItem(IDC_RADIO_ACTUAL))->SetCheck(TRUE);
	}
	else
	{
		enable = false;
		((CButton *)GetDlgItem(IDC_RADIO_PERCENT))->SetCheck(TRUE);
	}

	GetDlgItem(IDC_EDT_ACTUAL)->EnableWindow(enable);
	GetDlgItem(IDC_EDT_PERCENT)->EnableWindow(!enable);
	GetDlgItem(IDC_SPN_PERCENT)->EnableWindow(!enable);
}

void Raminfo::UpdateFrameNumber()
{
	frameNumber = MemoryManager::GetInstance()->CalculateFrameNumber();
	UpdateData(FALSE);
}

void Raminfo::SetActualEditText()
{
	CString text;
	text.Format("%.2f", m_fActual);
	GetDlgItem(IDC_EDT_ACTUAL)->SetWindowText(text);
}

BOOL Raminfo::PreTranslateMessage(MSG* pMsg)
{
	if ((pMsg->wParam == VK_ESCAPE))
	{
		return TRUE;
	}

	if ((pMsg->wParam == VK_RETURN))
	{
		UpdateData();
		if (GetFocus()->GetDlgCtrlID() == IDC_EDT_ACTUAL)
		{
			m_fActual = (float)max(min(m_fActual, MemoryManager::GetInstance()->GetAvailablePhysicalMemoryGB()), 0.01);
			MemoryManager::GetInstance()->SetActualPhysicalMemoryGB(m_fActual);
			UpdateUI();
		}
		return TRUE;
	}

	return CDialog::PreTranslateMessage(pMsg);
}

void Raminfo::OnBnClickedRadioActual()
{
	MemoryManager::GetInstance()->SetMemorySetMode(MemoryManager::ACTUAL);
	UpdateUI();
}

void Raminfo::OnBnClickedRadioPercent()
{
	MemoryManager::GetInstance()->SetMemorySetMode(MemoryManager::PERCENT);
	UpdateUI();
}

void Raminfo::OnEnChangeEdtPercent()
{
	UpdateData();
	m_nPercent = max(min(m_nPercent, 100), 1);
	MemoryManager::GetInstance()->SetPhysicalMemoryPercent(m_nPercent);
	UpdateFrameNumber();
}

void Raminfo::OnDeltaposSpnPercent(NMHDR *pNMHDR, LRESULT *pResult)
{
	LPNMUPDOWN pNMUpDown = reinterpret_cast<LPNMUPDOWN>(pNMHDR);
	if (pNMUpDown->iDelta < 0)
	{
		if (m_nPercent >= 100)
		{
			return;
		}
		m_nPercent += 1;
	}
	else if (pNMUpDown->iDelta > 0)
	{
		if (m_nPercent == 0)
		{
			return;
		}
		m_nPercent -= 1;     
	}
	UpdateData(FALSE);
	OnEnChangeEdtPercent();
	*pResult = 0;
}

void Raminfo::OnBnClickedOk()
{
	UpdateData();
	MemoryManager::GetInstance()->SetActualPhysicalMemoryGB(m_fActual);
	MemoryManager::GetInstance()->SetPhysicalMemoryPercent(m_nPercent);
	///ImageSaveManager::GetInstance()->AllocateRAMBufList();
	CDialog::OnOK();
}
