#ifndef ROLLINGSCANMANAGER_H
#define ROLLINGSCANMANAGER_H

#include "../sdk/inc/TUCamApi.h"
#include <map>
#include <vector>
using namespace std;

class RollingScanManager
{
public:
    enum ROLLING_SCAN_MODE
    {
        OFF = 0,
        LINE_TIME_DELAY = 1,
        SLIT_HEIGHT = 2,
    };

    enum READOUT_DIRECTION_TYPE
    {
        DOWN = 0,
        UP = 1,
        DOWN_UP_CYCLE = 2,
    };

    enum READOUT_DIRECTION_RESET_FLAG
    {
        YES = 0,
        NO = 1,
    };
    static RollingScanManager *GetInstance();
    ~RollingScanManager();
	void SetCameraHandle(HDTUCAM h);
    void Init();

    bool IsSupportRollingScan();

    void GetRollingScanModeList(vector<char *> &list);
    int GetRollingScanMode() { return rollingScanMode; }
    void SetRollingScanMode(int mode);

	void UpdateLineTimeDelayRange();
    void GetLineTimeDelayRange(int *minValue, int *maxValue);
    int GetMinLineTimeDelay() { return lineTimeDelayMin; }
    int GetMaxLineTimeDelay() { return lineTimeDelayMax; }
	int GetStepLineTimeDelay() { return lineTimeDelayStep; }
    int GetLineTimeDelay() { return lineTimeDelay; }
    void SetLineTimeDelay(int value);

	void UpdateSlitHeightRange();
    void GetSlitHeightRange(int *minValue, int *maxValue);
    int GetMinSlitHeight() { return slitHeightMin; }
    int GetMaxSlitHeight() { return slitHeightMax; }
	int GetStepSlitHeight() { return slitHeightStep; }
    int GetSlitHeight() { return slitHeight; }
    void SetSlitHeight(int value);

    double GetLineIntervalTime() { return lineIntervalTime; }

    void GetReadoutDirectionList(vector<char *> &list);
    int GetReadoutDirection() { return readoutDirection; }
    void SetReadoutDirection(int dir);

    void GetReadoutDirectionResetList(vector<char *> &list);
    int GetReadoutDirectionReset() { return readoutDirectionReset; }
    void SetReadoutDirectionReset(int value);

    void SetDefaultRollingScanParameter();
    int GetSlitHeightSingleStep();

private:
    static RollingScanManager *instance;
    HDTUCAM cameraHandle;
    int rollingScanMode;

    int lineTimeDelay;
    int lineTimeDelayMin;
    int lineTimeDelayMax;
	int lineTimeDelayStep;

    int slitHeight;
    int slitHeightMin;
    int slitHeightMax;
	int slitHeightStep;

    double lineIntervalTime;

    int readoutDirection;
    int readoutDirectionReset;

    map<int, vector<char *>> m_mapText;

    RollingScanManager();

    void GetCapaRange(int capaID, int *minValue, int *maxValue);
    void GetCapaTextList(vector<char *> &list, int capaID);
    int ConvertLineTimeDelaySlitHeight(int value);
    double CalculateLineIntervalTime(int delay);

    int GetCapaDefaultValue(int capaID);

    class CGarbo 
    {
    public:
        ~CGarbo()
        {
            if (RollingScanManager::instance)
            {
                delete RollingScanManager::instance;
                RollingScanManager::instance = NULL;
            }
                
        }
    };

    static CGarbo Garbo; 
};

#endif // ROLLINGSCANMANAGER_H
